-------------------------------------------------
--                                             --
--                  Mind's eye                 --
--                                             --
--          See beyound your two eyes!         --
--                                             --
-------------------------------------------------


-- Variables -----------------------------------------------------------

RWConfig.MindEye = {

	Active = true,
	
	Location = { x = 400, y = 400 },

	BarFontSize = 16,
	BarWidth = 180,	
	BarHeight = 26,
	
	BarBackColor = { r = 0, g = 0, b = 0, a = 0.7 },
	BarHealthColor = { r = 0.3, g = 0.7, b = 0.3, a = 1 },
	BarShieldColor = { r = 0.1, g = 0.1, b = 1, a = 0.5 },
	
	EnnemyTextColor = { r = 0.9, g = 0.0, b = 0.0, a = 1 },
	NeutralTextColor = { r = 1, g = 1, b = 0.1, a = 1 },
	AllyTextColor = { r = 1, g = 1, b = 1, a = 1 },	
	DeadTextColor = { r = 0.5, g = 0.5, b = 0.5, a = 1 },
	LostTextColor = { r = 0.8, g = 0.8, b = 0.8, a = 1 },
	LostBarColor = { r = 0.5, g = 0.5, b = 0.5 },
	
	BarSeparation = 2,
	NewBarDown = false,

	SelectThikness = 2,
	SelectColor = { r = 1, g = 1, b = 1, a = 1 },
	
	DotTTL = 2.2,
	DotMinWarnTime = 7.5,
	DotDeadWarnTime = 2,
	DotDeadFlashTime = 0.25,
	
	DotPopLeft = true,
	DotSize = 24,
	DotSeparation = 4,
	DotBarSep = 6,
	
	FadeTarget = true,
	FadeFocus = true,
	FadingValue = 0.4,
	
	SelectEnabled = true,
	
	useConfigHelp = true
}

RWConst.MindEye = {
	HBarSep = 1,
	UpdateTTL = 3,
	BarFontHeightRatio = 0.6
}

RWData.MindEye = {

	DeadText = RWRoot.locTxt.MindEye.UnitDeadText,
	MovingFrameText = RWRoot.locTxt.MovingFrameText,
	
	BarDY = 0,
	DotDX = 0,
	DotX = 0,
	DotY = 0,
	DotX2 = 0,
	DotY2 = 0,
	DotLimit = 0,
	HBarWidth = 0,
	HBarHeight = 0,
	FlashCycle = 0,
	FocusAlpha = 0,
	TargetAlpha = 0,
}

RWRoot.MindEye = {

	ConfigWindow = nil,
	MindEyeFrame = nil	
}

local ZombieBar = {}
local ZombieDot = {}

local DotInfo = {
	unknown = ""
}

local IconPath = {
	xicon = "resources\\xicon.png",
	up = "resources\\upicon.png", 
	down = "resources\\downicon.png",
	left = "resources\\lefticon.png",
	right = "resources\\righticon.png",
	updown = "resources\\updownicon.png",
	leftright = "resources\\leftrighticon.png"
}


-- Dot Icon object --------------------------------------------------------------------------------------------------


local function CreateDotIcon(Parent, abilityID, index)

	local DotIcon = UI.CreateFrame("Texture", "DotIcon", Parent)
	DotIcon:SetVisible(false)

	DotIcon.AbilityID = abilityID
	DotIcon.Index = index
	
	DotIcon.CheckTime = 0
	DotIcon.DeadTime = 0
	DotIcon.WarnTime = 0
	DotIcon.flashStatus = 0
	
	--== load icon image
	function DotIcon:LoadIcon()
		local infoID = self.AbilityID
		local source = "Rift"
		if infoID then
			if not DotInfo[infoID] then
				local detail = Inspect.Ability.New.Detail(infoID)
				if detail then
					DotInfo[infoID] = detail.icon
				else
					infoID = "unknown"
					source = RWData.AddonIdentifier
				end	
			end
		else
			infoID = "unknown"
			source = RWData.AddonIdentifier			
		end
		self:SetTexture(source, DotInfo[infoID])
	end
	
	--== Init Data
	function DotIcon:InitData()
			
		self.CheckTime = 0
		self.DeadTime = 0
		self.WarnTime = 0
		self.flashStatus = 0
		self:LoadIcon()
	end
	
	--== Update times!
	function DotIcon:UpdateID()

		self.CheckTime = RWRoot.CurrentTime + RWConfig.MindEye.DotTTL
		self.DeadTime = self.CheckTime + RWConfig.MindEye.DotDeadWarnTime
		if self.WarnTime == 0 then self.WarnTime = RWRoot.CurrentTime + RWConfig.MindEye.DotMinWarnTime end
		if self.flashStatus > 0 then
			self:LoadIcon()			
			self.flashStatus = 0
		end
	end
	
	--== Update status!
	function DotIcon:Update()
	
		if RWRoot.CurrentTime < self.CheckTime then return true end
		
		if RWRoot.CurrentTime < self.WarnTime then return false end
	
		if RWRoot.CurrentTime < self.DeadTime then
			local v = bit.band(math.floor(RWRoot.CurrentTime * RWData.MindEye.FlashCycle), 0x01)
			if self.flashStatus ~= v then
				self.flashStatus = v
				if v == 0 then
					self:LoadIcon()
				else
					self:SetTexture(RWData.AddonIdentifier, IconPath.xicon)
				end
			end
			return true
		else
			return false 
		end		
	end
	
	--== Set position
	function DotIcon:SetPosition()
		local dotparent = self:GetParent()
		local x = self.Index * RWData.MindEye.DotDX
		self:ClearAll()
		self:SetPoint( "TOPLEFT", dotparent, "TOPLEFT", RWData.MindEye.DotX + x , RWData.MindEye.DotY )	
		self:SetPoint( "BOTTOMRIGHT", dotparent, "BOTTOMLEFT", RWData.MindEye.DotX2 + x, RWData.MindEye.DotY2 )
	end
	
	--== Init Creation Data
	function DotIcon:InitCreationData()
	
	end

	--== Update config changes
	function DotIcon:UpdateChanges()
		self:SetPosition()
		self:InitCreationData()
	end
	

	DotIcon:InitData()
	DotIcon:SetPosition()
	
	return DotIcon
end


-- FocusBar object --------------------------------------------------------------------------------------------------


local function CreateFocusBar(Parent, focusID, keepFocus, Index)

	local FocusBar = UI.CreateFrame("Frame", "BarFrame", Parent)
	FocusBar:SetVisible(false)

	FocusBar:SetBackgroundColor( RWConfig.MindEye.BarBackColor.r, RWConfig.MindEye.BarBackColor.g, RWConfig.MindEye.BarBackColor.b, RWConfig.MindEye.BarBackColor.a )
	FocusBar.Health = UI.CreateFrame("Frame", "HealthFrame", FocusBar)
	FocusBar.Shield = UI.CreateFrame("Frame", "ShieldFrame", FocusBar)

	FocusBar.Health:SetPoint("TOPLEFT", FocusBar, "TOPLEFT", RWConst.MindEye.HBarSep, RWConst.MindEye.HBarSep)
	FocusBar.Shield:SetPoint("TOPLEFT", FocusBar, "TOPLEFT")
	
	FocusBar.FocusName = UI.CreateFrame("Text", "TargetName", FocusBar)
	FocusBar.FocusName:SetWidth(RWConfig.MindEye.BarWidth - 20 )
	FocusBar.FocusName:SetPoint("CENTERLEFT", FocusBar, "CENTERLEFT", 10, 0)
	
	FocusBar.Health:SetLayer(1)
	FocusBar.Shield:SetLayer(2)
	FocusBar.FocusName:SetLayer(3)
	
	FocusBar.KeepFocus = keepFocus
	FocusBar.FocusID = focusID
	FocusBar.MasterIndex = Index
	FocusBar.FocusRelation = 1
	
	FocusBar.Dead = false
	FocusBar.Shielded = false
	FocusBar.Lost = false
	FocusBar.UpdateNeeded = true
	FocusBar.UpdateLimit = 0
	
	FocusBar.isActive = true
	
	FocusBar.MaxDot = 0
	FocusBar.ActiveAbility = {}
	FocusBar.ActiveDot = {}

	--== The unit is dead!
	function FocusBar:SetDead()
		self.Dead = true
		self.Lost = false
		self:RemoveAllDot()
		self.FocusName:SetText(RWData.MindEye.DeadText)
		self.FocusName:SetFontColor( RWConfig.MindEye.DeadTextColor.r, RWConfig.MindEye.DeadTextColor.g, RWConfig.MindEye.DeadTextColor.b )
		self.Health:SetVisible(false)
		self.Shield:SetVisible(false)		
	end
	
	--== Update Focus Stats
	function FocusBar:UpdateInfo(unit)
		if not unit then unit = Inspect.Unit.Detail(self.FocusID) end
			
		-- unit in contact?
		if unit then -- yes
			self.UpdateNeeded = false
			self.UpdateLimit = RWRoot.CurrentTime + RWConst.MindEye.UpdateTTL
					
			-- Update Health and Shield
			if unit.health and (unit.health > 0) then
				self.Health:SetWidth(math.floor(RWData.MindEye.HBarWidth * (unit.health / unit.healthMax)))
				if self.Dead or self.Lost then self:InitData(unit) end
				if not unit.absorb then
					if self.Shielded then
						self.Shielded = false
						self.Shield:SetVisible(false)	
					end			
				else
					self.Shield:SetWidth(math.floor(RWConfig.MindEye.BarWidth * (unit.absorb / unit.healthMax)))
					if not self.Shielded then
						self.Shield:SetVisible(true)
						self.Shielded = true
					end
				end				
		
			elseif not self.Dead then -- unit is dead!
				self:SetDead()
			end
		
		elseif (not self.Lost) and (not self.Dead) then -- contact lost
			self.Lost = true
			self.FocusName:SetFontColor( RWConfig.MindEye.LostTextColor.r, RWConfig.MindEye.LostTextColor.g, RWConfig.MindEye.LostTextColor.b )
			self.Health:SetBackgroundColor( RWConfig.MindEye.LostBarColor.r, RWConfig.MindEye.LostBarColor.g, RWConfig.MindEye.LostBarColor.b, RWConfig.MindEye.BarHealthColor.a )
			self.Shield:SetBackgroundColor( RWConfig.MindEye.LostBarColor.r, RWConfig.MindEye.LostBarColor.g, RWConfig.MindEye.LostBarColor.b, RWConfig.MindEye.BarShieldColor.a )
		end		
	end
	
	--== Init Data
	function FocusBar:InitData(unit)
		if not unit then unit = Inspect.Unit.Detail(self.FocusID) end
		
		self.isActive = true
		self.Dead = false
		self.Shielded = false
		self.Health:SetVisible(true)
		self.Shield:SetVisible(false)
		
		if unit then
			if unit.relation == "friendly" then
				self.FocusRelation = 1
				if MTools.ClassColor[unit.calling] then
					self.FocusName:SetFontColor( MTools.ClassColor[unit.calling].r, MTools.ClassColor[unit.calling].g, MTools.ClassColor[unit.calling].b )
				else
					self.FocusName:SetFontColor( RWConfig.MindEye.AllyTextColor.r, RWConfig.MindEye.AllyTextColor.g, RWConfig.MindEye.AllyTextColor.b )
				end
			else --if unit.relation == "hostile" then 		-- neutral considered as hostile (for now), to avoid the status change check
				self.FocusRelation = 3
				self.FocusName:SetFontColor( RWConfig.MindEye.EnnemyTextColor.r, RWConfig.MindEye.EnnemyTextColor.g, RWConfig.MindEye.EnnemyTextColor.b )
	
			--else -- neutral
				--self.FocusRelation = 2
				--self.FocusName:SetFontColor( RWConfig.MindEye.NeutralTextColor.r, RWConfig.MindEye.NeutralTextColor.g, RWConfig.MindEye.NeutralTextColor.b )
			end
			self.FocusName:SetText(unit.name or "Unknown")
			self.Lost = false
			self.Health:SetBackgroundColor( RWConfig.MindEye.BarHealthColor.r, RWConfig.MindEye.BarHealthColor.g, RWConfig.MindEye.BarHealthColor.b, RWConfig.MindEye.BarHealthColor.a )
			self.Shield:SetBackgroundColor( RWConfig.MindEye.BarShieldColor.r, RWConfig.MindEye.BarShieldColor.g, RWConfig.MindEye.BarShieldColor.b, RWConfig.MindEye.BarShieldColor.a )
		
		else
			self.FocusName:SetText("Unknown")
			self.Lost = true
			self.FocusName:SetFontColor( RWConfig.MindEye.LostTextColor.r, RWConfig.MindEye.LostTextColor.g, RWConfig.MindEye.LostTextColor.b )
			self.Health:SetBackgroundColor( RWConfig.MindEye.LostBarColor.r, RWConfig.MindEye.LostBarColor.g, RWConfig.MindEye.LostBarColor.b, RWConfig.MindEye.BarHealthColor.a )
			self.Shield:SetBackgroundColor( RWConfig.MindEye.LostBarColor.r, RWConfig.MindEye.LostBarColor.g, RWConfig.MindEye.LostBarColor.b, RWConfig.MindEye.BarShieldColor.a )
		end
	end
	
	--== Resurect a dot or create a new one
	function FocusBar:AddNewDot(abilityID)

		self.MaxDot = self.MaxDot + 1
		
		-- resurecting
		local dot = table.remove(ZombieDot)
		if dot then
			dot:SetParent(self) -- adopted!^^
			dot.AbilityID = abilityID
			dot.Index = self.MaxDot
			dot:SetPosition()
			dot:InitData()
			
		else -- or creating
			dot = CreateDotIcon(self, abilityID, self.MaxDot)
		end

		self.ActiveAbility[abilityID] = dot
		self.ActiveDot[self.MaxDot] = dot		
		dot:SetVisible(true)
	end
		
	--== Remove a dot
	function FocusBar:RemoveDot(i)
	
		self.MaxDot = self.MaxDot - 1
		local dot = self.ActiveDot[i]
		dot:SetVisible(false)
		self.ActiveDot[i] = nil
		self.ActiveAbility[dot.AbilityID] = nil
		table.insert(ZombieDot, dot)
	end	
	
	--== Remove all dots!
	function FocusBar:RemoveAllDot()
		self.isActive = false
		while self.MaxDot > 0 do self:RemoveDot(self.MaxDot) end
	end
	
	--== Update an ability data
	function FocusBar:UpdateID(abilityID)
	
		if self.UpdateNeeded then self:UpdateInfo() end
		if not self.Dead then -- avoid asynchronous dot refresh!		
			if self.ActiveAbility[abilityID] then
				self.ActiveAbility[abilityID]:UpdateID()
			else
				self:AddNewDot(abilityID)
				self.ActiveAbility[abilityID]:UpdateID()
			end
		end
	end
	
	--== Update Data
	function FocusBar:Update()
	
		self.UpdateNeeded = true
		if RWRoot.CurrentTime > self.UpdateLimit then self:UpdateInfo() end -- no update since a long time, force it!
		
		-- Update Dots	
		local i = self.MaxDot
		local limit = self.MaxDot
		local j = 0 -- will hold the first nil index
		
		-- Remove inactive dots
		while i > 0 do
			if not self.ActiveDot[i]:Update() then
				self:RemoveDot(i)
				j = i
			end
			i = i - 1
		end
		
		if j > 0 then 
			if self.MaxDot == 0 then -- all dots were removed!
				self.isActive = false 
			else
				-- we lost some dots! clear the gaps!
				i = j + 1
				while i <= limit do						
					while (i <= limit) and (not self.ActiveDot[i]) do i = i + 1 end 
					if i > limit then break end
				
					self.ActiveDot[j] = self.ActiveDot[i]
					self.ActiveDot[j].Index = j
					self.ActiveDot[j]:SetPosition()
				
					self.ActiveDot[i] = nil
					j = j + 1
					i = i + 1
				
					while (j < limit) and (self.ActiveDot[j]) do j = j + 1 end
					if i <= j then i = j + 1 end
				end
			end
		end
		
		return (self.isActive or self.KeepFocus)
	end
	
	--== change keep status
	function FocusBar:SetKeepStatus(keep)
		self.KeepFocus = keep
		if self.MaxDot == 0 then self.isActive = false end
		if self.dead then
			if keep then
				self.DeathTimer = 0
				self:SetAlpha(1)
			else
				self.DeathTimer = RWRoot.CurrentTime + RWConfig.MindEye.DeathFadingTime
			end	
		end
	end
	
	--== Set FocusBar position
	function FocusBar:SetPosition()
		self:ClearPoint("TOPLEFT")
		self:SetPoint( "TOPLEFT", self:GetParent(), "TOPLEFT", 0, (self.MasterIndex - 1) * RWData.MindEye.BarDY )	
	end	
	
	--== Init Creation Data
	function FocusBar:InitCreationData()
		self:SetWidth(RWConfig.MindEye.BarWidth)
		self:SetHeight(RWConfig.MindEye.BarHeight)
		self.Health:SetHeight(RWData.MindEye.HBarHeight)
		self.Shield:SetHeight(RWConfig.MindEye.BarHeight)		
		self.FocusName:SetFontSize(RWConfig.MindEye.BarFontSize)	
	end

	--== Update Config changes
	function FocusBar:UpdateChanges()
		self:SetPosition()
		self:InitCreationData()		
		for i = 1, self.MaxDot do
			self.ActiveDot[i]:UpdateChanges()
		end		
	end
	
	-- Initialisation
	FocusBar:InitCreationData()
	FocusBar:InitData()
	FocusBar:SetPosition()
	
	return FocusBar
end


-- MasterFrame Object -------------------------------------------------------------------------------------------------


local function CreateMasterFrame()

	local MasterFrame = UI.CreateFrame("Frame", "MasterFrame", RWRoot.context)
	MasterFrame:SetVisible(false)
	
	MasterFrame.SelectFrame = MTools.GetRectangleFrame("MindEyeSelect", MasterFrame, RWConfig.MindEye.BarWidth + RWConfig.MindEye.SelectThikness * 2,
						RWConfig.MindEye.BarHeight + RWConfig.MindEye.SelectThikness * 2, RWConfig.MindEye.SelectThikness, RWConfig.MindEye.SelectColor)
	MasterFrame.SelectFrame:SetVisible(false)	
	
	MasterFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.MindEye.Location.x, RWConfig.MindEye.Location.y)
	
	MasterFrame.SelectedID = nil
	MasterFrame.SavedFocusID = nil
	
	MasterFrame.MaxBar = 0
	MasterFrame.ActiveFocus = {}
	MasterFrame.ActiveBar = {}
		
	-- Update selection position
	function MasterFrame:UpdateSelectPos()
		if RWConfig.MindEye.SelectEnabled then
			self.SelectFrame:ClearPoint("TOPLEFT")
			self.SelectFrame:SetPoint( "TOPLEFT", self, "TOPLEFT", -RWConfig.MindEye.SelectThikness, 
									((self.ActiveFocus[self.SelectedID].MasterIndex - 1) * RWData.MindEye.BarDY) - RWConfig.MindEye.SelectThikness )	
		end
	end
	
	-- Resurect a bar or create a new one
	function MasterFrame:AddNewBar(focusID, keepFocus)

		self.MaxBar = self.MaxBar + 1
		
		-- resurecting
		local bar = table.remove(ZombieBar)
		if bar then 
			bar:SetAlpha(1)
			bar.KeepFocus = keepFocus
			bar.FocusID = focusID
			bar.UpdateLimit = 0
			if self.MaxBar ~= bar.MasterIndex then
				bar.MasterIndex = self.MaxBar
				bar:SetPosition()
			end
			bar:InitData()
			
		else -- or creating
			bar = CreateFocusBar(self, focusID, keepFocus, self.MaxBar)
		end

		self.ActiveFocus[focusID] = bar
		self.ActiveBar[self.MaxBar] = bar		
		bar:SetVisible(true)
	end
	
	-- Remove a focus bar
	function MasterFrame:RemoveBar(i)
	
		self.MaxBar = self.MaxBar - 1
		local bar = self.ActiveBar[i]
		bar:SetVisible(false)
		bar:RemoveAllDot() -- just to be sure, if we are removing the bar, no dot should be there!
		self.ActiveBar[i] = nil
		self.ActiveFocus[bar.FocusID] = nil
		if self.SelectedID == bar.FocusID then
			self.SelectFrame:SetVisible(false)
			self.SelectedID = nil		
		end
		if self.SavedFocusID == bar.FocusID then self.SavedFocusID = nil end
		table.insert(ZombieBar, bar)
	end

	--== Remove all bars!
	function MasterFrame:RemoveAllBar()
		while self.MaxBar > 0 do self:RemoveBar(self.MaxBar) end	
	end
	
	-- Update damage info on a focus bar
	function MasterFrame:UpdateID(info)
		if info.target and info.abilityNew then
		
			if self.ActiveFocus[info.target] then
				self.ActiveFocus[info.target]:UpdateID(info.abilityNew)
				return
			end
		
			self:AddNewBar(info.target, false)
			self.ActiveFocus[info.target]:UpdateID(info.abilityNew)
		end
	end
	
	-- Update info on all bars
	function MasterFrame:Update()
				
		local i = self.MaxBar
		local limit = self.MaxBar
		local j = 0 -- will hold the first nil index
		-- Remove inactive bars
		while i > 0 do
			if not self.ActiveBar[i]:Update() then
				self:RemoveBar(i)
				j = i
			end
			i = i - 1
		end
		
		if j > 0 then
			-- we lost some bars! clear the gaps!
			i = j + 1	
			while i <= limit do						
				while (i <= limit) and (not self.ActiveBar[i]) do i = i + 1 end 
				if i > limit then break end

				self.ActiveBar[j] = self.ActiveBar[i]
				self.ActiveBar[j].MasterIndex = j
				self.ActiveBar[j]:SetPosition()
				if self.SelectedID == self.ActiveBar[j].FocusID then self:UpdateSelectPos() end -- selection moved too!
				self.ActiveBar[i] = nil
				
				j = j + 1
				i = i + 1
				
				while (j < limit) and (self.ActiveBar[j]) do j = j + 1 end
				if i <= j then i = j + 1 end
			end
		end
			
		-- update focus		
		if (self.SavedFocusID ~= RWRoot.focusId) and RWConfig.MindEye.FadeFocus then 
			if RWRoot.focusId and self.ActiveFocus[RWRoot.focusId] then
				if self.SavedFocusID then self.ActiveFocus[self.SavedFocusID]:SetAlpha(1) end
				self.ActiveFocus[RWRoot.focusId]:SetAlpha(RWConfig.MindEye.FadingValue) 
				self.SavedFocusID = RWRoot.focusId
				
			elseif self.SavedFocusID then
				self.ActiveFocus[self.SavedFocusID]:SetAlpha(1)
				self.SavedFocusID = nil
			end
		end		
		
		-- Update selection (target)
		if self.SelectedID ~= RWRoot.targetId then
			if RWRoot.targetId and self.ActiveFocus[RWRoot.targetId] then
				if RWConfig.MindEye.FadeTarget then	
					self.ActiveFocus[RWRoot.targetId]:SetAlpha(RWConfig.MindEye.FadingValue) 
					if self.SelectedID then
						if self.SelectedID ~= self.SavedFocusID then 
							self.ActiveFocus[self.SelectedID]:SetAlpha(1) 
						else
							self.ActiveFocus[self.SelectedID]:SetAlpha(RWData.MindEye.FocusAlpha) 
						end
					end
				end
				self.SelectedID = RWRoot.targetId
				self:UpdateSelectPos()
				self.SelectFrame:SetVisible(RWConfig.MindEye.SelectEnabled)
				
			elseif self.SelectedID then
				self.SelectFrame:SetVisible(false)
				if self.SelectedID ~= self.SavedFocusID then 
					self.ActiveFocus[self.SelectedID]:SetAlpha(1) 
				else
					self.ActiveFocus[self.SelectedID]:SetAlpha(RWData.MindEye.FocusAlpha) 
				end
				self.SelectedID = nil
			end
		end
		
	end
	
	-- Adding a permanent focus
	function MasterFrame:KeepPlayerTarget()
	
		if RWRoot.targetId then
		if self.ActiveFocus[RWRoot.targetId] then
			-- Target already exist! easy, we need to change it keep status only
			self.ActiveFocus[RWRoot.targetId]:SetKeepStatus(true)
		else
			-- Target dont exists! the hard work starts
			-- if the target is an ally we put it before the hostile and neutral focus
			local j = 1
			while (j <= self.MaxBar) and (self.ActiveBar[j].FocusRelation == 1) do j = j + 1 end -- first non ally index
			
			self:AddNewBar(RWRoot.targetId, true)
			
			local i = self.MaxBar
			if (self.ActiveBar[i].FocusRelation ~= 1) or (i == j) then return end -- the focus is not an ally! or we have only allies! .. leaving!
			
			-- Shifting bars
			local bar = self.ActiveBar[i]
			while i > j do
				self.ActiveBar[i] = self.ActiveBar[i - 1]
				self.ActiveBar[i].MasterIndex = i
				self.ActiveBar[i]:SetPosition()
				i = i - 1			
			end
			-- Adding the ally to his rightful position!
			self.ActiveBar[i] = bar
			self.ActiveBar[i].MasterIndex = i
			self.ActiveBar[i]:SetPosition()
			-- we are selecting this, but the selection will be refreshed during the next frame update, no need to do it twice
		end			
		end
	end

	-- removing a permanent focus
	function MasterFrame:RemovePlayerTarget()
		if RWRoot.targetId and self.ActiveFocus[RWRoot.targetId] then 
			self.ActiveFocus[RWRoot.targetId]:SetKeepStatus(false) -- why removing is always easier than adding! destroying is easier than creating!^^
		end	
	end

	-- removing all keeped focus
	function MasterFrame:unkeepAll()
		for i = 1, self.MaxBar do
			if self.ActiveBar[i].KeepFocus then self.ActiveBar[i]:SetKeepStatus(false) end
		end
	end
	
	-- update the changes made using the config window!
	function MasterFrame:UpdateChanges()
		if self.SelectedID then 
			self.SelectFrame:SetVisible(RWConfig.MindEye.SelectEnabled)
			self:UpdateSelectPos()
			if RWConfig.MindEye.FadeTarget then
				self.ActiveFocus[self.SelectedID]:SetAlpha(RWConfig.MindEye.FadingValue)			
			else
				self.ActiveFocus[self.SelectedID]:SetAlpha(1)
			end					
		end
		
		if self.SavedFocusID then
			if RWConfig.MindEye.FadeFocus then
				self.ActiveFocus[self.SavedFocusID]:SetAlpha(RWConfig.MindEye.FadingValue)			
			else
				if self.SavedFocusID ~= self.SelectedID then self.ActiveFocus[self.SavedFocusID]:SetAlpha(1) end
			end		
		end

		for i = 1, self.MaxBar do
			self.ActiveBar[i]:UpdateChanges()
		end	
		
		-- update the zombie bars & dots now so we dont have to do it when resurecting them
		for _, v in pairs(ZombieBar) do
			v:InitCreationData()
		end
		for _, v in pairs(ZombieDot) do
			v:InitCreationData()
		end	
	end
	
	return MasterFrame
end


-- Init functions --------------------------------------------------------------------------------------------------

function RWRoot.MindEye.InitData()

	RWData.MindEye.BarDY = RWConfig.MindEye.BarHeight + RWConfig.MindEye.BarSeparation
	if not RWConfig.MindEye.NewBarDown then RWData.MindEye.BarDY = -RWData.MindEye.BarDY end
	RWData.MindEye.DotLimit = RWConfig.MindEye.DotTTL + RWConfig.MindEye.DotDeadWarnTime
	RWData.MindEye.HBarWidth = RWConfig.MindEye.BarWidth - RWConst.MindEye.HBarSep * 2
	RWData.MindEye.HBarHeight = RWConfig.MindEye.BarHeight - RWConst.MindEye.HBarSep * 2
	RWData.MindEye.FlashCycle = 1/RWConfig.MindEye.DotDeadFlashTime
	RWData.MindEye.DotDX = RWConfig.MindEye.DotSize + RWConfig.MindEye.DotSeparation
	RWData.MindEye.DotY = math.floor((RWConfig.MindEye.BarHeight - RWConfig.MindEye.DotSize) / 2)
	RWData.MindEye.DotY2 = RWData.MindEye.DotY + RWConfig.MindEye.DotSize - RWConfig.MindEye.BarHeight
	if not RWConfig.MindEye.DotPopLeft then
		RWData.MindEye.DotX = RWConfig.MindEye.BarWidth + RWConfig.MindEye.DotBarSep
	else
		RWData.MindEye.DotX =  -RWConfig.MindEye.DotSize - RWConfig.MindEye.DotBarSep
		RWData.MindEye.DotDX = -RWData.MindEye.DotDX
	end
	RWData.MindEye.DotX = RWData.MindEye.DotX - RWData.MindEye.DotDX -- dot position is indexed from 1, not 0, so DotX start at -1!
	RWData.MindEye.DotX2 = RWData.MindEye.DotX + RWConfig.MindEye.DotSize
	RWData.MindEye.FocusAlpha = 1
	RWData.MindEye.TargetAlpha = 1
	if RWConfig.MindEye.FadeFocus then RWData.MindEye.FocusAlpha = RWConfig.MindEye.FadingValue end
	if RWConfig.MindEye.FadeTarget then RWData.MindEye.TargetAlpha = RWConfig.MindEye.FadingValue end
end


-- Activate/Desactivate -----------------------------------------------------------

function RWRoot.MindEye.InitAddon()

	RWRoot.MindEye.InitData()
	
	if not RWRoot.MindEye.MindEyeFrame then RWRoot.MindEye.MindEyeFrame = CreateMasterFrame() end
	RWRoot.MindEye.MindEyeFrame:SetVisible(true)
	
end

-- Sleep
function RWRoot.MindEye.Sleep()
	if RWConfig.MindEye.Active then
	
	if RWRoot.MindEye.MindEyeFrame then
		RWRoot.MindEye.MindEyeFrame:SetVisible(false)
		RWRoot.MindEye.MindEyeFrame:RemoveAllBar()
	end
	
	end
end

-- Awake
function RWRoot.MindEye.Awake()
	if RWConfig.MindEye.Active then
	
	RWRoot.MindEye.InitAddon()
	
	end
end

-- Activate
function RWRoot.MindEye.Activate(ForceActivation)
	if (not RWConfig.MindEye.Active) or ForceActivation then
		RWConfig.MindEye.Active = true
		RWRoot.MindEye.InitAddon()	
		print(RWRoot.locTxt.MindEye.ModActivated)
	end
end

-- Desactivate
function RWRoot.MindEye.Desactivate()
	if RWConfig.MindEye.Active then
	
	RWRoot.MindEye.Sleep()
	RWConfig.MindEye.Active = false
	print(RWRoot.locTxt.MindEye.ModDesactivated)
	
	end
end


-- Config window -----------------------------------------------------------


-- Helper window
local HelperConst = {
	startX = 4,
	textsep = 2,
	fontSize = 18
}

local function BuildPopupHelpFrame(parent)

	local HelperFrame = UI.CreateFrame("Frame", "MindEyeFrameConfig", parent)
	HelperFrame:SetVisible(false)

	HelperFrame:SetBackgroundColor( 0.2, 0.2, 0.2, 0.5)
	
	HelperFrame.Line1 = UI.CreateFrame("Text", "HelpLine1", HelperFrame) 
	HelperFrame.Line2 = UI.CreateFrame("Text", "HelpLine2", HelperFrame) 
	HelperFrame.Line3 = UI.CreateFrame("Text", "HelpLine3", HelperFrame) 
	
	HelperFrame.Line1:SetFontSize(HelperConst.fontSize - 4)
	HelperFrame.Line2:SetFontSize(HelperConst.fontSize - 4)
	HelperFrame.Line3:SetFontSize(HelperConst.fontSize - 4)
	
	HelperFrame.Line1:SetPoint("TOPLEFT", HelperFrame, "TOPLEFT", HelperConst.startX, HelperConst.startX )
	HelperFrame.Line2:SetPoint("TOPLEFT", HelperFrame, "TOPLEFT", HelperConst.startX, HelperConst.startX + HelperConst.fontSize + HelperConst.textsep)
	HelperFrame.Line3:SetPoint("TOPLEFT", HelperFrame, "TOPLEFT", HelperConst.startX, HelperConst.startX + (HelperConst.fontSize + HelperConst.textsep) * 2)
	
	function HelperFrame:ShowHelp(text1, text2, text3)
		if text1 then
			HelperFrame.Line1:SetText(text1)
			local width = HelperFrame.Line1:GetWidth()
			local height = HelperConst.startX * 2 + HelperConst.fontSize + 2
			if text2 then
				HelperFrame.Line2:SetText(text2)
				if width < HelperFrame.Line2:GetWidth() then width = HelperFrame.Line2:GetWidth() end
				height = height + HelperConst.fontSize + HelperConst.textsep
				HelperFrame.Line2:SetVisible(true)
				if text3 then
					HelperFrame.Line3:SetText(text3)
					if width < HelperFrame.Line3:GetWidth() then width = HelperFrame.Line3:GetWidth() end
					height = height + HelperConst.fontSize + HelperConst.textsep
					HelperFrame.Line3:SetVisible(true)				
				else
					HelperFrame.Line3:SetVisible(false)
				end 				
			else
				HelperFrame.Line2:SetVisible(false)
				HelperFrame.Line3:SetVisible(false)
			end
			HelperFrame:SetWidth(width + HelperConst.startX * 2)
			HelperFrame:SetHeight(height)
			HelperFrame:SetVisible(true)
		end
	end
	
	return HelperFrame	
end


-- Building a config master frame!

local function BuildConfigFrame()

	local ConfigFrame = UI.CreateFrame("Frame", "MindEyeFrameConfig", RWRoot.ConfigWinManager)
	ConfigFrame:SetVisible(false)
	
	ConfigFrame.HelpFrame = BuildPopupHelpFrame(ConfigFrame)
	ConfigFrame.usehelp = true
	
	ConfigFrame:SetBackgroundColor( RWConfig.MindEye.BarBackColor.r, RWConfig.MindEye.BarBackColor.g, RWConfig.MindEye.BarBackColor.b, RWConfig.MindEye.BarBackColor.a )
	ConfigFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.MindEye.Location.x, RWConfig.MindEye.Location.y)
	MTools.makeMovable(ConfigFrame)
		
	ConfigFrame.Text = UI.CreateFrame("Text", "MindEyeFramePos", ConfigFrame)
	ConfigFrame.Text:SetText(RWData.MindEye.MovingFrameText)
	ConfigFrame.Text:SetPoint("CENTER", ConfigFrame, "CENTER")

	ConfigFrame.ResizeIcon = UI.CreateFrame("Texture", "ResizeIcon", ConfigFrame)	
	ConfigFrame.DotPopIcon = UI.CreateFrame("Texture", "DotPopIcon", ConfigFrame)
	ConfigFrame.BarPopIcon = UI.CreateFrame("Texture", "BarPopIcon", ConfigFrame)

	ConfigFrame.ResizeIcon:SetWidth(24)
	ConfigFrame.ResizeIcon:SetHeight(24)
	
	ConfigFrame.ResizeIcon:SetPoint("CENTERLEFT", ConfigFrame, "CENTERLEFT", 8, 0)
	
	ConfigFrame.DotPopIcon:SetBackgroundColor( RWConfig.MindEye.BarBackColor.r, RWConfig.MindEye.BarBackColor.g, RWConfig.MindEye.BarBackColor.b, RWConfig.MindEye.BarBackColor.a )
	ConfigFrame.BarPopIcon:SetBackgroundColor( RWConfig.MindEye.BarBackColor.r, RWConfig.MindEye.BarBackColor.g, RWConfig.MindEye.BarBackColor.b, RWConfig.MindEye.BarBackColor.a )
	
	ConfigFrame.ResizeHeight = true
	ConfigFrame.PointLeft = true
	ConfigFrame.PointDown = true
	
	
	-- ResizeIcon
	function ConfigFrame:LoadResizeIcon()
		if ConfigFrame.ResizeHeight then
			ConfigFrame.ResizeIcon:SetTexture(RWData.AddonIdentifier, IconPath.updown)
		else
			ConfigFrame.ResizeIcon:SetTexture(RWData.AddonIdentifier, IconPath.leftright)
		end	
	end
	
	function ConfigFrame.ResizeIcon.Event:LeftDown()
		ConfigFrame.ResizeHeight = not ConfigFrame.ResizeHeight
		ConfigFrame:LoadResizeIcon()	
	end
	
	-- BarPopIcon
	function ConfigFrame:UpdateBarPopIcon()
		MTools.ClearPoints(ConfigFrame.BarPopIcon)
		if ConfigFrame.PointDown then
			ConfigFrame.BarPopIcon:SetTexture(RWData.AddonIdentifier, IconPath.down)
			ConfigFrame.BarPopIcon:SetPoint("TOPCENTER", ConfigFrame, "BOTTOMCENTER", 0, RWConfig.MindEye.DotBarSep)
		else
			ConfigFrame.BarPopIcon:SetTexture(RWData.AddonIdentifier, IconPath.up)
			ConfigFrame.BarPopIcon:SetPoint("TOPCENTER", ConfigFrame, "TOPCENTER", 0, -ConfigFrame.BarPopIcon:GetHeight() - RWConfig.MindEye.DotBarSep)
		end
	end

	function ConfigFrame.BarPopIcon.Event:LeftDown()
		ConfigFrame.PointDown = not ConfigFrame.PointDown
		ConfigFrame:UpdateBarPopIcon()	
	end
	
	function ConfigFrame.BarPopIcon.Event:MouseIn()
		if not ConfigFrame.usehelp then return end
		MTools.ClearPoints(ConfigFrame.HelpFrame)
		if ConfigFrame.PointDown then		
			ConfigFrame.HelpFrame:SetPoint("TOPLEFT", ConfigFrame.BarPopIcon, "TOPRIGHT", 10, 0)
		else
			ConfigFrame.HelpFrame:SetPoint("BOTTOMLEFT", ConfigFrame.BarPopIcon, "BOTTOMRIGHT", 10, 0)
		end 
		ConfigFrame.HelpFrame:ShowHelp(RWRoot.locTxt.MindEye.hlpBarPopIcon)
	end

	function ConfigFrame.BarPopIcon.Event:MouseOut()
		ConfigFrame.HelpFrame:SetVisible(false)
	end

	-- DotPopIcon
	function ConfigFrame:SetDotPopIconPos()
		MTools.ClearPoints(ConfigFrame.DotPopIcon)
		if ConfigFrame.PointLeft then
			ConfigFrame.DotPopIcon:SetPoint("CENTERLEFT", ConfigFrame, "CENTERLEFT", -ConfigFrame.DotPopIcon:GetWidth() - RWConfig.MindEye.DotBarSep, 0)
		else
			ConfigFrame.DotPopIcon:SetPoint("CENTERLEFT", ConfigFrame, "CENTERRIGHT", RWConfig.MindEye.DotBarSep, 0)
		end	
	end
	
	function ConfigFrame:UpdateDotPopIcon()
		if ConfigFrame.PointLeft then
			ConfigFrame.DotPopIcon:SetTexture(RWData.AddonIdentifier, IconPath.left)
		else
			ConfigFrame.DotPopIcon:SetTexture(RWData.AddonIdentifier, IconPath.right)
		end
		ConfigFrame:SetDotPopIconPos()
	end

	function ConfigFrame.DotPopIcon.Event:LeftDown()
		ConfigFrame.PointLeft = not ConfigFrame.PointLeft
		ConfigFrame:UpdateDotPopIcon()	
	end
		
	function ConfigFrame.DotPopIcon.Event:RightDown()
		ConfigFrame.DotPopIcon:SetWidth(ConfigFrame:GetHeight())
		ConfigFrame.DotPopIcon:SetHeight(ConfigFrame:GetHeight())
		ConfigFrame:SetDotPopIconPos()	
		ConfigFrame.BarPopIcon:SetWidth(ConfigFrame:GetHeight())
		ConfigFrame.BarPopIcon:SetHeight(ConfigFrame:GetHeight())
		ConfigFrame:UpdateBarPopIcon()	
	end	
	
	function ConfigFrame.DotPopIcon.Event:WheelForward()
		ConfigFrame.DotPopIcon:SetWidth(ConfigFrame.DotPopIcon:GetWidth() + 1)
		ConfigFrame.DotPopIcon:SetHeight(ConfigFrame.DotPopIcon:GetWidth())
		ConfigFrame:SetDotPopIconPos()	
		ConfigFrame.BarPopIcon:SetWidth(ConfigFrame.DotPopIcon:GetWidth())
		ConfigFrame.BarPopIcon:SetHeight(ConfigFrame.DotPopIcon:GetWidth())
		ConfigFrame:UpdateBarPopIcon()	
	end		
	
	function ConfigFrame.DotPopIcon.Event:WheelBack()
		ConfigFrame.DotPopIcon:SetWidth(ConfigFrame.DotPopIcon:GetWidth() - 1)
		ConfigFrame.DotPopIcon:SetHeight(ConfigFrame.DotPopIcon:GetWidth())
		ConfigFrame:SetDotPopIconPos()
		ConfigFrame.BarPopIcon:SetWidth(ConfigFrame.DotPopIcon:GetWidth())
		ConfigFrame.BarPopIcon:SetHeight(ConfigFrame.DotPopIcon:GetWidth())
		ConfigFrame:UpdateBarPopIcon()	
	end
	
	function ConfigFrame.DotPopIcon.Event:MouseIn()
		if not ConfigFrame.usehelp then return end
		MTools.ClearPoints(ConfigFrame.HelpFrame)
		if ConfigFrame.PointLeft then		
			ConfigFrame.HelpFrame:SetPoint("TOPRIGHT", ConfigFrame.DotPopIcon, "TOPLEFT", -10, 0)
		else
			ConfigFrame.HelpFrame:SetPoint("TOPLEFT", ConfigFrame.DotPopIcon, "TOPRIGHT", 10, 0)
		end
		ConfigFrame.HelpFrame:ShowHelp(
			RWRoot.locTxt.MindEye.hlpDotPopIcon1,
			RWRoot.locTxt.MindEye.hlpDotPopIcon2,
			RWRoot.locTxt.MindEye.hlpDotPopIcon3)
	end
	
	function ConfigFrame.DotPopIcon.Event:MouseOut()
		ConfigFrame.HelpFrame:SetVisible(false)
	end
	
	-- main bar
	function ConfigFrame.Event:WheelForward()
		if ConfigFrame.ResizeHeight then
			ConfigFrame:SetHeight(ConfigFrame:GetHeight() + 1)
			ConfigFrame.Text:SetFontSize(math.floor(ConfigFrame:GetHeight() * RWConst.MindEye.BarFontHeightRatio))			
		else
			ConfigFrame:SetWidth(ConfigFrame:GetWidth() + 1)		
		end
	end		
	
	function ConfigFrame.Event:WheelBack()
		if ConfigFrame.ResizeHeight then
			ConfigFrame:SetHeight(ConfigFrame:GetHeight() - 1)
			ConfigFrame.Text:SetFontSize(math.floor(ConfigFrame:GetHeight() * RWConst.MindEye.BarFontHeightRatio))			
		else
			ConfigFrame:SetWidth(ConfigFrame:GetWidth() - 1)		
		end
	end
	
	function ConfigFrame.Event:MouseIn()
		if not ConfigFrame.usehelp then return end
		MTools.ClearPoints(ConfigFrame.HelpFrame)
		if ConfigFrame:GetLeft() < 600 then
			ConfigFrame.HelpFrame:SetPoint("TOPLEFT", ConfigFrame, "BOTTOMRIGHT", 10, 10)
		else
			ConfigFrame.HelpFrame:SetPoint("TOPRIGHT", ConfigFrame, "BOTTOMLEFT", -10, 10)	
		end
		ConfigFrame.HelpFrame:ShowHelp(
			RWRoot.locTxt.MindEye.hlpUnitBar1,
			RWRoot.locTxt.MindEye.hlpUnitBar2,
			RWRoot.locTxt.MindEye.hlpUnitBar3)
	end
	
	function ConfigFrame.Event:MouseOut()
		ConfigFrame.HelpFrame:SetVisible(false)
	end
	

	function ConfigFrame:SetHelpStatus(help)
		ConfigFrame.usehelp = help
		if not help then ConfigFrame.HelpFrame:SetVisible(false) end
	end
	
	-- Initialisation
	function ConfigFrame:LoadData()
		ConfigFrame.PointLeft = RWConfig.MindEye.DotPopLeft
		ConfigFrame.PointDown = RWConfig.MindEye.NewBarDown
		ConfigFrame.DotPopIcon:SetWidth(RWConfig.MindEye.DotSize)
		ConfigFrame.DotPopIcon:SetHeight(RWConfig.MindEye.DotSize)
		ConfigFrame.BarPopIcon:SetWidth(RWConfig.MindEye.DotSize)
		ConfigFrame.BarPopIcon:SetHeight(RWConfig.MindEye.DotSize)	
		
		ConfigFrame:SetWidth(RWConfig.MindEye.BarWidth)
		ConfigFrame:SetHeight(RWConfig.MindEye.BarHeight)
		
		ConfigFrame.Text:SetFontSize(RWConfig.MindEye.BarFontSize)
		
		ConfigFrame.usehelp = RWConfig.MindEye.useConfigHelp
	
		ConfigFrame:UpdateDotPopIcon()
		ConfigFrame:UpdateBarPopIcon()		
	end	
	
	function ConfigFrame:SaveData()
		RWConfig.MindEye.DotPopLeft = ConfigFrame.PointLeft
		RWConfig.MindEye.NewBarDown = ConfigFrame.PointDown
		RWConfig.MindEye.DotSize = ConfigFrame.DotPopIcon:GetWidth()
		
		RWConfig.MindEye.BarWidth = ConfigFrame:GetWidth()
		RWConfig.MindEye.BarHeight = ConfigFrame:GetHeight()
		
		RWConfig.MindEye.BarFontSize = math.floor(ConfigFrame:GetHeight() * RWConst.MindEye.BarFontHeightRatio)		
	end	
	
	ConfigFrame:LoadResizeIcon()
	
	return ConfigFrame
end


-- building the config window

local function BuildConfigWindow()

	local window = RWRoot.ConfigWinManager:CreateWindow()
	window:SetVisible(false)
	
	window.ConfigFramePos = BuildConfigFrame()	
	
	window:SetTitle(RWRoot.locTxt.MindEye.ModuleName)
	window:SetHeight(520)
	window:SetWidth(400)
	window:SetPoint("CENTER", UIParent, "CENTER")
	
	MTools.buildWindowMover("ConfigWindow", window)
		
	-- boutons
	window.bbClose =  MTools.buildButton("btClose", window, 50, 50, "x", "close")
	window.bbClose:SetPoint("TOPRIGHT", window, "TOPRIGHT", -8, 15)

	window.btOk =  MTools.buildButton("btOk", window, 110, 40, RWRoot.locTxt.buttonOk)
	window.btOk:SetPoint("TOPLEFT", window, "TOPLEFT", 90, 440)

	window.btCancel =  MTools.buildButton("btAnnuler", window, 110, 40, RWRoot.locTxt.buttonCancel)
	window.btCancel:SetPoint("TOPLEFT", window, "TOPLEFT", 200, 440)

	-- Options des barres
	window.ttBarOptionsTitle = MTools.buildShadowTitle("ttBarOptionsTitle", window, 360, 25, RWRoot.locTxt.MindEye.cfgBarTitle)
	window.ttBarOptionsTitle:SetPoint("TOPLEFT", window, "TOPLEFT", 20, 70)

	window.ckSelectEnabled = MTools.buildCheckBox("ckSelectEnabled", window, 360, 20, RWRoot.locTxt.MindEye.cfgShowSelection)
	window.ckSelectEnabled:SetPoint("TOPLEFT", window.ttBarOptionsTitle, "TOPLEFT", 10, 30)
	
	window.ckFadeTarget = MTools.buildCheckBox("ckFadeTarget", window, 360, 20, RWRoot.locTxt.MindEye.cfgFadeTarget)
	window.ckFadeTarget:SetPoint("TOPLEFT", window.ckSelectEnabled, "TOPLEFT", 0, 30)
	
	window.ckFadeFocus = MTools.buildCheckBox("ckFadeFocus", window, 360, 20, RWRoot.locTxt.MindEye.cfgFadeFocus)
	window.ckFadeFocus:SetPoint("TOPLEFT", window.ckFadeTarget, "TOPLEFT", 0, 30)
	
	window.slFadingValue = MTools.buildIntegerSlider("slFadingValue", window, 340, 20, RWRoot.locTxt.MindEye.cfgFadingValue, 5, 0, 100, "%")
	window.slFadingValue:SetPoint("TOPLEFT", window.ckFadeFocus, "TOPLEFT", 0, 50)	

	-- optins des icones
	window.ttIconOptionsTitle = MTools.buildShadowTitle("ttIconOptionsTitle", window, 360, 25, RWRoot.locTxt.MindEye.cfgIconTitle)
	window.ttIconOptionsTitle:SetPoint("TOPLEFT", window, "TOPLEFT", 20, 240)

	window.slDotMinWarnTime = MTools.buildRealSlider("slDotMinWarnTime", window, 340, 20, RWRoot.locTxt.MindEye.cfgIconMinWTime, 7.5, 0, 15, 1, " sec")
	window.slDotMinWarnTime:SetPoint("TOPLEFT", window.ttIconOptionsTitle, "TOPLEFT", 10, 55)

	window.slDotDeadWarnTime = MTools.buildRealSlider("slDotDeadWarnTime", window, 340, 20, RWRoot.locTxt.MindEye.cfgIconWarnTime, 1, 0.0, 5, 1, " sec")
	window.slDotDeadWarnTime:SetPoint("TOPLEFT", window.slDotMinWarnTime, "TOPLEFT", 0, 50)

	window.slDotDeadFlashTime = MTools.buildRealSlider("slDotDeadFlashTime", window, 340, 20, RWRoot.locTxt.MindEye.cfgIconFlashValue, 1, 0.1, 1, 2, " sec")
	window.slDotDeadFlashTime:SetPoint("TOPLEFT", window.slDotDeadWarnTime, "TOPLEFT", 0, 50)

	
	window.ckuseConfigHelp = MTools.buildCheckBox("ckuseConfigHelp", window, 360, 17, RWRoot.locTxt.MindEye.cfgShowHelp)
	window.ckuseConfigHelp:SetPoint("TOPLEFT", window, "TOPLEFT", 20, 483)
	
	
	function window:LoadData()
		MTools.ClearPoints(window.ConfigFramePos)
		window.ConfigFramePos:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.MindEye.Location.x, RWConfig.MindEye.Location.y)		
		window.ConfigFramePos:LoadData()
		window.ConfigFramePos:SetVisible(true)
		
		window.ckSelectEnabled:SetChecked(RWConfig.MindEye.SelectEnabled)
		window.ckFadeTarget:SetChecked(RWConfig.MindEye.FadeTarget)
		window.ckFadeFocus:SetChecked(RWConfig.MindEye.FadeFocus)
		window.slFadingValue:SetValue(math.floor(RWConfig.MindEye.FadingValue*100))
	
		window.slDotMinWarnTime:SetRealValue(RWConfig.MindEye.DotMinWarnTime)
		window.slDotDeadWarnTime:SetRealValue(RWConfig.MindEye.DotDeadWarnTime)
		window.slDotDeadFlashTime:SetRealValue(RWConfig.MindEye.DotDeadFlashTime)

		window.ckuseConfigHelp:SetChecked(RWConfig.MindEye.useConfigHelp)
	end
	
	function window:SaveData()
		RWConfig.MindEye.Location.x = window.ConfigFramePos:GetLeft()
		RWConfig.MindEye.Location.y = window.ConfigFramePos:GetTop()

		window.ConfigFramePos:SaveData()
	
		if RWRoot.MindEye.MindEyeFrame then
			MTools.ClearPoints(RWRoot.MindEye.MindEyeFrame)
			RWRoot.MindEye.MindEyeFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.MindEye.Location.x, RWConfig.MindEye.Location.y)
		end		
		
		RWConfig.MindEye.SelectEnabled = window.ckSelectEnabled:GetChecked()
		RWConfig.MindEye.FadeTarget = window.ckFadeTarget:GetChecked()
		RWConfig.MindEye.FadeFocus = window.ckFadeFocus:GetChecked()
		RWConfig.MindEye.FadingValue = window.slFadingValue:GetValue()/100
	
		RWConfig.MindEye.DotMinWarnTime = window.slDotMinWarnTime:GetRealValue()
		RWConfig.MindEye.DotDeadWarnTime = window.slDotDeadWarnTime:GetRealValue()
		RWConfig.MindEye.DotDeadFlashTime = window.slDotDeadFlashTime:GetRealValue()
		
		RWConfig.MindEye.useConfigHelp = window.ckuseConfigHelp:GetChecked()

		RWRoot.MindEye.InitData()	
		if RWRoot.MindEye.MindEyeFrame then RWRoot.MindEye.MindEyeFrame:UpdateChanges() end		
		print(RWRoot.locTxt.MindEye.cfgOptionsSaved)
	end
	
	function window.ckuseConfigHelp.Event:CheckboxChange()
		window.ConfigFramePos:SetHelpStatus(window.ckuseConfigHelp:GetChecked())
	end
	
	function window:CloseWindow()
		window.ConfigFramePos:SetVisible(false)
		window:SetVisible(false)
	end
	
	function window.bbClose.Event:LeftPress()
		RWRoot.ConfigWinManager:CloseWindow()
	end
	
	function window.btCancel.Event:LeftPress()
		window.bbClose.Event:LeftPress()		
	end
	
	function window.btOk.Event:LeftPress()
		window:SaveData()
		window.bbClose.Event:LeftPress()		
	end

	RWRoot.MindEye.ConfigWindow = window
	return window
end


--	Button registration ----------------------

RWRoot.HermesButton.MainButton:RegisterMenuButton(RWRoot.locTxt.MindEye.abTitle, 
	"mindeyecfg", RWRoot.locTxt.MindEye.abConfig, {
		"Resources\\Button\\btnMindeyeCfgUp.png",
		"Resources\\Button\\btnMindeyeCfgIn.png",
		"Resources\\Button\\btnMindeyeCfgDown.png"}):RegisterModButton(
	"keep", RWRoot.locTxt.MindEye.abKeep, {
		"Resources\\Button\\MindEyeKeepUp.png",
		"Resources\\Button\\MindEyeKeepIn.png",
		"Resources\\Button\\MindEyeKeepDown.png"}):RegisterModButton(
	"forget", RWRoot.locTxt.MindEye.abForget, {
		"Resources\\Button\\MindEyeForgetUp.png",
		"Resources\\Button\\MindEyeForgetIn.png",
		"Resources\\Button\\MindEyeForgetDown.png"}):RegisterModButton(
	"forgetall", RWRoot.locTxt.MindEye.abForgetAll, {
		"Resources\\Button\\MinDEyeForgetAllUp.png",
		"Resources\\Button\\MinDEyeForgetAllIn.png",
		"Resources\\Button\\MinDEyeForgetAllDown.png"})


--	Config win registration ----------------------

RWRoot.ConfigWinManager:RegisterWindow("MindEyeCfgWin", BuildConfigWindow)

