-------------------------------------------------
--                                             --
--              Proximity Warning              --
--                                             --
--      Are you too close to other players?    --
--                                             --
-------------------------------------------------


-- Variables -----------------------------------------------------------

RWConfig.ProxyWarn = {

	Active = false,
	
	Location = { x = 300, y = 300 },

	FrameWidth = 160,	
	FrameColor = { r = 0, g = 0, b = 0, a = 0.4 },
	TitleColor = { r = 1, g = 1, b = 1, a = 1 },
	FontSize = 18,
	NameSeparation = 1,
	
	IngoreOutLOS = false,
	SortData = false,
	MaxPlayerShown = 5,
	ShowDistance = false,
	
	MinDistance = 5.0*5.0,
	WarningDistance = 3.5*3.5,
	
	CheckPerFrame = 7	
}

RWConst.ProxyWarn = {
}

RWData.ProxyWarn = {
	FrameTitle = RWRoot.locTxt.ProxyWarn.FrameTitle,
	MovingFrameText = RWRoot.locTxt.MovingFrameText
}

RWRoot.ProxyWarn = {
	ConfigWindow = nil,
	ProxyFrame = nil,
	
	NextPlayer = 1
}

local ProxyPlayer = {}


-- Init functions -----------------------------------------------------------

local function GetProxyFrameHeight()
	return (RWConfig.ProxyWarn.FontSize + RWConfig.ProxyWarn.NameSeparation) * (RWConfig.ProxyWarn.MaxPlayerShown + 1) + 12
end

local function CreateProxyFrame()

	local Frame = UI.CreateFrame("Frame", "ProxyFrame", RWRoot.context)
	Frame:SetVisible(false)
	
	Frame:SetWidth(RWConfig.ProxyWarn.FrameWidth)
	Frame:SetHeight( GetProxyFrameHeight() )
	Frame:SetBackgroundColor( RWConfig.ProxyWarn.FrameColor.r, RWConfig.ProxyWarn.FrameColor.g, RWConfig.ProxyWarn.FrameColor.b, RWConfig.ProxyWarn.FrameColor.a )
	Frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.ProxyWarn.Location.x, RWConfig.ProxyWarn.Location.y)
	
	Frame.Title = UI.CreateFrame("Text", "ProxyTitle", Frame)
	Frame.Title:SetPoint("TOPCENTER", Frame, "TOPCENTER")
	Frame.Title:SetFontSize(RWConfig.ProxyWarn.FontSize)
	Frame.Title:SetFontColor( RWConfig.ProxyWarn.TitleColor.r, RWConfig.ProxyWarn.TitleColor.g, RWConfig.ProxyWarn.TitleColor.b, RWConfig.ProxyWarn.TitleColor.a )
	Frame.Title:SetText(RWData.ProxyWarn.FrameTitle)
	
	Frame.MaxWarnText = 0
	Frame.proxyWarnText = {}

	Frame.ActualMax = 0
	
	function Frame:Init()
		Frame:SetVisible(false)
		for i = 1, Frame.ActualMax do Frame.proxyWarnText[i]:SetVisible(false) end		
		
		Frame.ActualMax = 0
	end
	
	function Frame:AdjustToNewMax()		
		if Frame.ActualMax > RWConfig.ProxyWarn.MaxPlayerShown then
			for i = RWConfig.ProxyWarn.MaxPlayerShown + 1, Frame.ActualMax do Frame.proxyWarnText[i]:SetVisible(false) end	
			Frame.ActualMax = RWConfig.ProxyWarn.MaxPlayerShown
		end		
		Frame:SetHeight( GetProxyFrameHeight() )	
	end
	
	function Frame:CreateProxyWarnText()		
		local FrameText = UI.CreateFrame("Text", "ProxyWarnText", Frame)
		FrameText:SetVisible(false)
		Frame.MaxWarnText = Frame.MaxWarnText + 1
		FrameText:SetPoint("TOPCENTER", Frame, "TOPCENTER", 0, 6 + Frame.MaxWarnText * (RWConfig.ProxyWarn.FontSize + RWConfig.ProxyWarn.NameSeparation) )
		FrameText:SetFontSize(RWConfig.ProxyWarn.FontSize)
		FrameText.NotThatClose = true
		
		Frame.proxyWarnText[Frame.MaxWarnText] = FrameText
	end
	
	function Frame:UpdateUnsorted()
		local i = 0
		for k, v in pairs(ProxyPlayer) do
			i = i + 1		
			if i > Frame.MaxWarnText then Frame:CreateProxyWarnText() end
			
			Frame.proxyWarnText[i]:SetVisible(false)
			local s = k
			if RWConfig.ProxyWarn.ShowDistance then 
				local x = math.sqrt(v.d)
				local x10 = math.floor(x * 10) % 10
				s = s .. " - " .. tostring(math.floor(x)) .. "." .. tostring(x10) .. "m"
			end
			Frame.proxyWarnText[i]:SetText(s)
			Frame.proxyWarnText[i]:SetFontColor(MTools.GetClassColor(v.calling))
			Frame.proxyWarnText[i].NotThatClose = v.d > RWConfig.ProxyWarn.WarningDistance
			if Frame.proxyWarnText[i].NotThatClose or RWRoot.FlashVisible()
			then Frame.proxyWarnText[i]:SetVisible(true) end
			
			if i == RWConfig.ProxyWarn.MaxPlayerShown then break end
		end
		
		for j = i + 1, Frame.ActualMax do
			Frame.proxyWarnText[j]:SetVisible(false)
		end
		
		if Frame.ActualMax ~= i then
			if Frame.ActualMax == 0 then Frame:SetVisible(true)
			elseif i == 0 then Frame:SetVisible(false) end

			Frame.ActualMax = i
		end
	end
	
	function Frame:UpdateSorted(playerList)
		local i = 0
		for k, v in ipairs(playerList) do
			i = i + 1		
			if i > Frame.MaxWarnText then Frame:CreateProxyWarnText() end
			
			Frame.proxyWarnText[i]:SetVisible(false)
			local s = v.name
			if RWConfig.ProxyWarn.ShowDistance then 
				local x = math.sqrt(v.d)
				local x10 = math.floor(x * 10) % 10
				s = s .. " - " .. tostring(math.floor(x)) .. "." .. tostring(x10) .. "m"
			end
			Frame.proxyWarnText[i]:SetText(s)
			Frame.proxyWarnText[i]:SetFontColor(MTools.GetClassColor(v.calling))
			Frame.proxyWarnText[i].NotThatClose = v.d > RWConfig.ProxyWarn.WarningDistance
			if Frame.proxyWarnText[i].NotThatClose or RWRoot.FlashVisible()
			then Frame.proxyWarnText[i]:SetVisible(true) end
			
			if i == RWConfig.ProxyWarn.MaxPlayerShown then break end
		end
		
		for j = i + 1, Frame.ActualMax do
			Frame.proxyWarnText[j]:SetVisible(false)
		end
		
		if Frame.ActualMax ~= i then
			if Frame.ActualMax == 0 then Frame:SetVisible(true)
			elseif i == 0 then Frame:SetVisible(false) end

			Frame.ActualMax = i
		end
	end

	function Frame:Update()
		for i = 1, Frame.ActualMax do
			if not Frame.proxyWarnText[i].NotThatClose then
				if RWRoot.FlashVisible() then
					Frame.proxyWarnText[i]:SetVisible(true) 
				else
					Frame.proxyWarnText[i]:SetVisible(false) 
				end
			end
		end
	end
	
	return Frame
end


function RWRoot.ProxyWarn.InitData()
	
	ProxyPlayer = {}	
	RWRoot.ProxyWarn.NextPlayer = 1
	
	if RWRoot.ProxyWarn.ProxyFrame then RWRoot.ProxyWarn.ProxyFrame:Init() end
end


function RWRoot.ProxyWarn.InitAddon()

	RWRoot.ProxyWarn.InitData()
	if not RWRoot.ProxyWarn.ProxyFrame then RWRoot.ProxyWarn.ProxyFrame = CreateProxyFrame() end
end



-- Activate/Desactivate -----------------------------------------------------------

-- Activate
function RWRoot.ProxyWarn.Activate(ForceActivation)
	if (not RWConfig.ProxyWarn.Active) or ForceActivation then
		RWConfig.ProxyWarn.Active = true
		RWRoot.ProxyWarn.InitAddon()	
		print(RWRoot.locTxt.ProxyWarn.ModActivated)
	end
end

-- Desactivate
function RWRoot.ProxyWarn.Desactivate()
	if RWConfig.ProxyWarn.Active then
	
	RWRoot.ProxyWarn.InitData()
	RWConfig.ProxyWarn.Active = false
	print(RWRoot.locTxt.ProxyWarn.ModDesactivated)
	
	end
end


-- Handlers -----------------------------------------------------------

function RWRoot.ProxyWarn.Test()
	local unitList = Inspect.Unit.List()
	for _, v in pairs(unitList) do
		local unit = Inspect.Unit.Detail(v)
		
		print (unit.name)
	
	end
	dump(unitList)
end

local function SortAndUpdateProxyFrame()

	local PPlayer = {}
	
	for k, v in pairs(ProxyPlayer) do
		v.name = k
		table.insert(PPlayer, v)
	end
	
	table.sort(PPlayer, function (a, b) return a.d < b.d end)
	
	RWRoot.ProxyWarn.ProxyFrame:UpdateSorted(PPlayer)    	
end


function RWRoot.ProxyWarn.Update()
	
	-- Check players
	if RWRoot.player and RWRoot.player.coordX then
		local i = RWRoot.ProxyWarn.NextPlayer
		local j = RWConfig.ProxyWarn.CheckPerFrame
	
		while j > 0 do
			RWRoot.CheckMember(i)
			local detail = RWRoot.RaidMember[i]
			if detail and (detail.id ~= RWRoot.playerId) and detail.coordX and (not (RWConfig.ProxyWarn.IngoreOutLOS and detail.blocked)) then
				local dx = detail.coordX - RWRoot.player.coordX
				local dy = detail.coordY - RWRoot.player.coordY
				local dz = detail.coordZ - RWRoot.player.coordZ
				local distance = dx * dx + dy * dy + dz * dz
			
				-- Is the player near us?
				if distance <= RWConfig.ProxyWarn.MinDistance then
					ProxyPlayer[detail.name] = { d = distance, calling = detail.calling }
				end
		
			end
			
			j = j - 1
			i = i + 1
			if i == 21 then 
				-- all players checked! update the frame!
				if not RWConfig.ProxyWarn.SortData then 
					RWRoot.ProxyWarn.ProxyFrame:UpdateUnsorted()
				else
					SortAndUpdateProxyFrame()
				end
				ProxyPlayer = {}
				RWRoot.ProxyWarn.NextPlayer = 1
				RWRoot.CheckPlayer()
				return
			end
		end
	
		RWRoot.ProxyWarn.NextPlayer = i
	end
	
	RWRoot.ProxyWarn.ProxyFrame:Update() -- update for the flashing text	
end


-- Config window -----------------------------------------------------------


local function BuildConfigWindow()

	local window = RWRoot.ConfigWinManager:CreateWindow()
	window:SetVisible(false)
	
	window.ConfigFramePos = UI.CreateFrame("Frame", "ProxyFramePos", RWRoot.ConfigWinManager)
	window.ConfigFramePos:SetVisible(false)
	
	window.ConfigFramePos:SetWidth(RWConfig.ProxyWarn.FrameWidth)
	window.ConfigFramePos:SetHeight(GetProxyFrameHeight())
	window.ConfigFramePos:SetBackgroundColor( RWConfig.ProxyWarn.FrameColor.r, RWConfig.ProxyWarn.FrameColor.g, RWConfig.ProxyWarn.FrameColor.b, RWConfig.ProxyWarn.FrameColor.a )
	window.ConfigFramePos:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.ProxyWarn.Location.x, RWConfig.ProxyWarn.Location.y)
	MTools.makeMovable(window.ConfigFramePos)
	
	window.ConfigFramePos.Text = UI.CreateFrame("Text", "ProxyFramePos", window.ConfigFramePos)
	window.ConfigFramePos.Text:SetText(RWData.ProxyWarn.MovingFrameText)
	window.ConfigFramePos.Text:SetFontSize(RWConfig.ProxyWarn.FontSize)
	window.ConfigFramePos.Text:SetPoint("CENTER", window.ConfigFramePos, "CENTER")

	
	window:SetTitle(RWRoot.locTxt.ProxyWarn.ModuleName)
	window:SetHeight(570)
	window:SetWidth(400)
	window:SetPoint("CENTER", UIParent, "CENTER")
	
	MTools.buildWindowMover("ConfigWindow", window)
		
	-- boutons
	window.bbClose =  MTools.buildButton("btClose", window, 50, 50, "x", "close")
	window.bbClose:SetPoint("TOPRIGHT", window, "TOPRIGHT", -8, 15)

	window.btOk =  MTools.buildButton("btOk", window, 110, 40, RWRoot.locTxt.buttonOk)
	window.btOk:SetPoint("BOTTOMLEFT", window, "BOTTOMLEFT", 90, -30)

	window.btCancel =  MTools.buildButton("btAnnuler", window, 110, 40, RWRoot.locTxt.buttonCancel)
	window.btCancel:SetPoint("BOTTOMRIGHT", window, "BOTTOMRIGHT", -90, -30)

	-- options
	window.ttOptionsTitle = MTools.buildShadowTitle("ttOptionsTitle", window, 360, 25, RWRoot.locTxt.Options)
	window.ttOptionsTitle:SetPoint("TOPLEFT", window, "TOPLEFT", 20, 70)
	
	window.slMinDistance = MTools.buildIntegerSlider("slMinDistance", window, 340, 20, RWRoot.locTxt.ProxyWarn.cfgMinDistance, 5, 1, 30)
	window.slMinDistance:SetPoint("TOPLEFT", window.ttOptionsTitle, "TOPLEFT", 10, 60)
	
	window.slWarningDistance = MTools.buildIntegerSlider("slWarningDistance", window, 340, 20, RWRoot.locTxt.ProxyWarn.cfgWarnDistance, 70, 1, 100, "%")
	window.slWarningDistance:SetPoint("TOPLEFT", window.slMinDistance, "TOPLEFT", 0, 50)

	window.lbWarningDistHelp = MTools.buildLabel("lbWarningDistHelp", window, 360, 12, RWRoot.locTxt.ProxyWarn.cfgWarnDistHelp)
	window.lbWarningDistHelp:SetPoint("BOTTOMLEFT", window.slWarningDistance, "BOTTOMLEFT", 0, 7)
	
	window.slMaxPlayerShown = MTools.buildIntegerSlider("slMaxPlayerShown", window, 340, 20, RWRoot.locTxt.ProxyWarn.cfgMaxPlayerShown, RWConfig.ProxyWarn.MaxPlayerShown, 4, 10)
	window.slMaxPlayerShown:SetPoint("TOPLEFT", window.slWarningDistance, "TOPLEFT", 0, 65)
	
	window.ckIngoreOutLOS = MTools.buildCheckBox("ckIngoreOutLOS", window, 360, 20, RWRoot.locTxt.ProxyWarn.cfgIngoreOutLOS)
	window.ckIngoreOutLOS:SetPoint("TOPLEFT", window.slMaxPlayerShown, "TOPLEFT", 0, 30)

	window.ckShowDistance = MTools.buildCheckBox("ckShowDistance", window, 360, 20, RWRoot.locTxt.ProxyWarn.cfgShowDistance)
	window.ckShowDistance:SetPoint("TOPLEFT", window.ckIngoreOutLOS, "TOPLEFT", 0, 30)

	
	-- performances
	window.ttPerfTitle = MTools.buildShadowTitle("ttPerfTitle", window, 360, 25, RWRoot.locTxt.Performance)
	window.ttPerfTitle:SetPoint("TOPLEFT", window, "TOPLEFT", 20, 345)
		
	window.slCheckPerFrame = MTools.buildIntegerSlider("slCheckPerFrame", window, 340, 20, RWRoot.locTxt.ProxyWarn.cfgCheckPerFrame, RWConfig.ProxyWarn.CheckPerFrame, 3, 10)
	window.slCheckPerFrame:SetPoint("TOPLEFT", window.ttPerfTitle, "TOPLEFT", 10, 60)
	
	window.lbBetterRefresh = MTools.buildText("lbBetterRefresh", window, RWRoot.locTxt.ProxyWarn.cfgBetterRefresh, 12)
	window.lbBetterRefresh:SetPoint("BOTTOMRIGHT", window.slCheckPerFrame, "BOTTOMRIGHT", 0, 7)
	
	window.lbBetterPerf = MTools.buildLabel("lbBetterPerf", window, 150, 12, RWRoot.locTxt.ProxyWarn.cfgBetterSpeed)
	window.lbBetterPerf:SetPoint("BOTTOMLEFT", window.slCheckPerFrame, "BOTTOMLEFT", 0, 7)

	window.ckSortData = MTools.buildCheckBox("ckSortData", window, 360, 20, RWRoot.locTxt.ProxyWarn.cfgSortList)
	window.ckSortData:SetPoint("TOPLEFT", window.slCheckPerFrame, "TOPLEFT", 0, 40)
	
	window.lbWarningDistHelp = MTools.buildLabel("lbWarningDistHelp", window, 360, 12, RWRoot.locTxt.ProxyWarn.cfgBetterSpdifOff)
	window.lbWarningDistHelp:SetPoint("BOTTOMLEFT", window.ckSortData, "BOTTOMLEFT", 20, 16)
		
		
	function window:LoadData()
		MTools.ClearPoints(window.ConfigFramePos)
		window.ConfigFramePos:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.ProxyWarn.Location.x, RWConfig.ProxyWarn.Location.y)
		window.ConfigFramePos:SetHeight(GetProxyFrameHeight())
		window.ConfigFramePos:SetVisible(true)
		window.ckIngoreOutLOS:SetChecked(RWConfig.ProxyWarn.IngoreOutLOS)
		window.ckShowDistance:SetChecked(RWConfig.ProxyWarn.ShowDistance)
		
		local mDist = math.sqrt(RWConfig.ProxyWarn.MinDistance)
		local wDist = math.floor((math.sqrt(RWConfig.ProxyWarn.WarningDistance) / mDist) * 100)
		
		window.slMinDistance:SetValue(math.floor(mDist))
		window.slWarningDistance:SetValue(wDist)
		window.slMaxPlayerShown:SetValue(RWConfig.ProxyWarn.MaxPlayerShown)
	
		window.slCheckPerFrame:SetValue(RWConfig.ProxyWarn.CheckPerFrame)
		window.ckSortData:SetChecked(RWConfig.ProxyWarn.SortData)
	end
	
	function window:SaveData()

		RWConfig.ProxyWarn.Location.x = window.ConfigFramePos:GetLeft()
		RWConfig.ProxyWarn.Location.y = window.ConfigFramePos:GetTop()
	
		if RWRoot.ProxyWarn.ProxyFrame then
			MTools.ClearPoints(RWRoot.ProxyWarn.ProxyFrame)
			RWRoot.ProxyWarn.ProxyFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.ProxyWarn.Location.x, RWConfig.ProxyWarn.Location.y)
		end		
		
		local mDist = window.slMinDistance:GetValue()
		local wDist = (window.slWarningDistance:GetValue() * mDist) / 100
		
		RWConfig.ProxyWarn.IngoreOutLOS = window.ckIngoreOutLOS:GetChecked()
		RWConfig.ProxyWarn.ShowDistance = window.ckShowDistance:GetChecked()
		RWConfig.ProxyWarn.MinDistance = mDist * mDist
		RWConfig.ProxyWarn.WarningDistance = wDist * wDist
		RWConfig.ProxyWarn.MaxPlayerShown = window.slMaxPlayerShown:GetValue()
		if RWRoot.ProxyWarn.ProxyFrame then RWRoot.ProxyWarn.ProxyFrame:AdjustToNewMax() end
	
		RWConfig.ProxyWarn.CheckPerFrame = window.slCheckPerFrame:GetValue()
		RWConfig.ProxyWarn.SortData = window.ckSortData:GetChecked()		
		
		print(RWRoot.locTxt.ProxyWarn.cfgOptionsSaved)
	end
	
	
	function window:CloseWindow()
		window.ConfigFramePos:SetVisible(false)
		window:SetVisible(false)
	end
	
	function window.bbClose.Event:LeftPress()
		RWRoot.ConfigWinManager:CloseWindow()
	end
	
	function window.btCancel.Event:LeftPress()
		window.bbClose.Event:LeftPress()		
	end
	
	function window.btOk.Event:LeftPress()
		window:SaveData()
		window.bbClose.Event:LeftPress()		
	end

	RWRoot.ProxyWarn.ConfigWindow = window
	return window
end


--	Button registration ----------------------

RWRoot.HermesButton.MainButton:RegisterMenuButton(RWRoot.locTxt.ProxyWarn.abTitle, 
	"proxycfg", RWRoot.locTxt.ProxyWarn.abConfig, {
		"Resources\\Button\\ProxyWarnCfgUp.png",
		"Resources\\Button\\ProxyWarnCfgIn.png",
		"Resources\\Button\\ProxyWarnCfgDown.png"})


--	Config win registration ----------------------

RWRoot.ConfigWinManager:RegisterWindow("ProxyWarnCfgWin", BuildConfigWindow)
