-------------------------------------------------
--                                             --
--                   MTools                    --
--                                             --
--          General functions library          --
--                                             --
-------------------------------------------------


-- Constantes / Variables -----------------------------------------------------------

MTools = {}

MTools.CosRange = 256 					-- must but a round binary number! 2^x
MTools.CosMask = MTools.CosRange - 1 	-- Cos Mask, if the rule above is respected it gives things like 0xFF
MTools.pi2 = 2 * math.pi
MTools.fcosarg = MTools.CosRange/MTools.pi2
MTools.CosTable = {}

MTools.ClassColor = {
	mage = { r = 0xc8/255.0, g = 0x5e/255.0, b = 0xff/255.0 },
	cleric = { r = 0x77/255.0, g = 0xef/255.0, b = 0x00/255.0 },
	warrior = { r = 0xff/255.0, g = 0x28/255.0, b = 0x28/255.0 },
	rogue = { r = 0xff/255.0, g = 0xdb/255.0, b = 0x00/255.0 }
}

MTools.RaidMember = {}
MTools.RaidMember[1] = "group01"
MTools.RaidMember[2] = "group02"
MTools.RaidMember[3] = "group03"
MTools.RaidMember[4] = "group04"
MTools.RaidMember[5] = "group05"
MTools.RaidMember[6] = "group06"
MTools.RaidMember[7] = "group07"
MTools.RaidMember[8] = "group08"
MTools.RaidMember[9] = "group09"
MTools.RaidMember[10] = "group10"
MTools.RaidMember[11] = "group11"
MTools.RaidMember[12] = "group12"
MTools.RaidMember[13] = "group13"
MTools.RaidMember[14] = "group14"
MTools.RaidMember[15] = "group15"
MTools.RaidMember[16] = "group16"
MTools.RaidMember[17] = "group17"
MTools.RaidMember[18] = "group18"
MTools.RaidMember[19] = "group19"
MTools.RaidMember[20] = "group20"

MTools.strIndex = {
	"01", "02", "03", "04", "05", "06", "07", "08",
	"09", "10", "11", "12", "13", "14", "15", "16",
	"17", "18", "19", "20", "21", "22", "23", "24",
	"25", "26", "27", "28", "29", "30", "31", "32"
}

MTools.bagIndex = {
	"si01.0", "si02.0", "si03.0", "si04.0", "si05.0", "si06.0", "si07.0"
}


MTools.context = nil
MTools.YesNoWindow = nil
MTools.TableString = ""
MTools.TSIndex = 0
MTools.TSLen = 0

-- Fast math! ----------------------------------------------------------------

local function fastCosInit()
	
	for i = 1, MTools.CosRange do
		MTools.CosTable[i] = math.cos((i-1)/MTools.fcosarg)
	end
end

--== return the arg to use to be able to call the fastest cos function (fcos(a))
function MTools.GetfCosArg(cycle)
	return MTools.CosRange/cycle
end

--== a is in [0, 2*pi[
function MTools.sfastCos(a)

	return MTools.CosTable[math.floor(a*MTools.fcosarg)+1]
end

--== a is in [0, infinity[ , but the cos range in [0, 2*pi[
function MTools.fastCos(a)

	return MTools.CosTable[bit.band(math.floor(a*MTools.fcosarg),MTools.CosMask)+1]
end

--== a is in [0, MTools.CosRange[
function MTools.sfCos(a)

	return MTools.CosTable[math.floor(a)+1]
end

--== a is in [0, infinity[ , but the cos range in [0, MTools.CosRange[
function MTools.fCos(a)
	return MTools.CosTable[bit.band(math.floor(a),MTools.CosMask)+1]
end


-------------------------------------------------------------------

function MTools.GetContext()
	if MTools.context then return MTools.context end
	MTools.context = UI.CreateContext("MTOOLSContext")
	return MTools.context
end

function MTools.SetContext(Context)
	if not MTools.context then
		MTools.context = Context
	end
end

function MTools.GetShortTimeStr(Time)
	local s = ""
	
	if Time >= 3600 then
		s = tostring(math.floor(Time/3600)) .. "h"
		return s
	end	
	if Time >= 100 then
		s = tostring(math.floor(Time/60)) .. "m"
		return s
	end	
	s = tostring(math.floor(Time)) .. "s"
	return s
end

function MTools.GetTimeString(Time)
	
	local TimeString = ""
	local MinuteFormat = "%d"
	
	if Time >= 3600 then 
		local hours = math.floor(Time/3600)
		TimeString = tostring(hours) .. ":"
		Time = Time - hours * 3600
		MinuteFormat = "%02d"
	end
	local minutes = math.floor(Time/60)
	TimeString = TimeString .. string.format(MinuteFormat, minutes) .. ":" .. string.format("%02d", math.floor(Time-minutes*60))
	
	return TimeString
end


-- Rift Related -----------------------------------------------------------------

-- get a class color
function MTools.GetClassColor(calling)
	if calling and MTools.ClassColor[calling] then return MTools.ClassColor[calling].r, MTools.ClassColor[calling].g, MTools.ClassColor[calling].b end
	return nil, nil, nil
end


-- Table operations -----------------------------------------------------------------

-- Copy a table
function MTools.CopyTable(t)

	local tCopy = {}
	for k, v in pairs(t) do
		if type(v) == "table" then
			tCopy[k] = MTools.CopyTable(t[k])
		else
			tCopy[k] = t[k]
		end
	end
	
	return tCopy
end

-- Copy all the data from source, but keep the data that destination have and source dont
function MTools.CopyData(destination, source)

	for k, _ in pairs(source) do
		if type(source[k]) == "table" then
			if type(destination[k]) == "table" then
				MTools.CopyData(destination[k], source[k])
			else
				destination[k] = MTools.CopyTable(source[k])		
			end
		else
			destination[k] = source[k]
		end
	end
end

-- Add new fields and replace the type modified one
function MTools.AddNewFields(destination, source)

	for k, _ in pairs(source) do
		
		if type(source[k]) == "table" then
			if type(destination[k]) == "table" then
				MTools.AddNewFields(destination[k], source[k])	
			else
				destination[k] = MTools.CopyTable(source[k])		
			end
			
		elseif type(destination[k]) ~= type(source[k]) then 
			destination[k] = source[k] 
			
		end
		
	end
end

-- remove unused fields and replace the type modified one
function MTools.RemoveUnusedFields(destination, source)

	for k, _ in pairs(destination) do	
	
		if type(source[k]) == "table" then
			if type(destination[k]) == "table" then 
				MTools.RemoveUnusedFields(destination[k], source[k])
			else
				destination[k] = MTools.CopyTable(source[k])			
			end
			
		elseif source[k] == nil then 
			destination[k] = nil
				
		end
	
	end
end

-- Adjust fields! copy new ones and remove unused ones
function MTools.AdjustFields(destination, source)
	MTools.AddNewFields(destination, source)
	MTools.RemoveUnusedFields(destination, source)
end

-- Remove empty table from the tree
function MTools.PurgeTable(t)

	local empty = true
	
	for k, v in pairs(t) do
		if type(v) == "table" then
			if MTools.PurgeTable(t[k]) then
				t[k] = nil
			else
				empty = false
			end
		else
			empty = false
		end
	end
	
	return empty
end

-- no boolean no indexed table allowed (for now)
function MTools.TableToString(t)
	local s = ""
	local ss = ""
	
	for k, v in pairs(t) do	
		s = s .. ss .. k .. " = "
		if type(v) == "table" then
			s = s .. "{" .. MTools.TableToString(t[k]) .. "}"
		elseif v == nil then 
			s = s .. "nil"
		elseif type(v) == "string" then
			s = s .. "\"" .. v .. "\""
		else
			s = s .. v
		end
		ss = ", "
	end
	
	return s
end

local function StrToTable()
	local t = {}
	local i = MTools.TSIndex
	
	while i <= MTools.TSLen do
		while (i <= MTools.TSLen) and (MTools.TableString:byte(i) == string.byte(" ")) do i = i + 1 end
		MTools.TSIndex = i
		if i > MTools.TSLen then break end
		
		if MTools.TableString:byte(i) == string.byte("}") then MTools.TSIndex = i + 1 break end
		if MTools.TableString:byte(i) == string.byte(",") then i = i + 2 MTools.TSIndex = i end
	
		while MTools.TableString:byte(i) ~= string.byte("=") do i = i + 1 end
		local k = string.sub(MTools.TableString, MTools.TSIndex, i-2)
		i = i + 2
	
		if MTools.TableString:byte(i) == string.byte("{") then
			MTools.TSIndex = i + 1
			t[k] = StrToTable()
			i = MTools.TSIndex
	
		elseif MTools.TableString:byte(i) == string.byte("\"") then
			i = i + 1
			MTools.TSIndex = i
			while MTools.TableString:byte(i) ~= string.byte("\"") do i = i + 1 end
			if MTools.TSIndex == i then
				t[k] = ""
			else
				t[k] = string.sub(MTools.TableString, MTools.TSIndex, i-1)
			end
			i = i + 1
			MTools.TSIndex = i
	
		else
			MTools.TSIndex = i
			while i <= MTools.TSLen
			and MTools.TableString:byte(i) ~= string.byte(",") 
			and MTools.TableString:byte(i) ~= string.byte("}") 
			do i = i + 1 end
			local xstr = string.sub(MTools.TableString, MTools.TSIndex, i-1)
			t[k] = tonumber(xstr) or xstr
			MTools.TSIndex = i
		end
	end
	
	return t
end

function MTools.StringToTable(s)
	MTools.TableString = s
	MTools.TSIndex = 1
	MTools.TSLen = string.len(s)
	return StrToTable()
end

--------------------------------------------------------

function MTools.InsideZone(x, y, x0, y0, dx, dy)
	return (x>=x0) and (x<(x0+dx)) and (y>=y0) and (y<(y0+dy))
end

function MTools.InsideSquare(x, y, x0, y0, size)
	return (x>=x0) and (x<(x0+size)) and (y>=y0) and (y<(y0+size))
end

-- Version operations! -----------------------------------------------------------------

function MTools.GetVersionNumbers(VerString)
	local ver = {
		high = 0,
		low = 0,
		fix = 0
	}
	
	if string.len(VerString) ~= 0 then
		local h, l = string.match(VerString,"(%w+).(%w+)")
		local f = VerString:byte(string.len(VerString))
		if (f >= string.byte("a")) and (f <= string.byte("z")) then
			ver.fix = f - string.byte("a") + 1
			l = string.sub(l, 1, string.len(l)-1)
		end
		ver.high = tonumber(h)
		ver.low = tonumber(l)
	end
	
	return ver
end

function MTools.GetVersionString(Ver)

	local version = tostring(Ver.high) .. "." .. string.format("%02d", Ver.low)
	
	if Ver.fix > 0 then
		version = version .. string.char( string.byte("a") + Ver.fix - 1 )
	end
	
	return version
end

function MTools.CheckLowerVersion(FinalVersion, SourceVersion)	
	-- Data exists?
	if not SourceVersion then return false
	elseif not FinalVersion then return true
	elseif not SourceVersion.high or not SourceVersion.low or not SourceVersion.fix then return false
	elseif not FinalVersion.high or not FinalVersion.low or not FinalVersion.fix then return true
	-- Yes!
	elseif FinalVersion.high < SourceVersion.high then return true
	elseif FinalVersion.high > SourceVersion.high then return false
	elseif FinalVersion.low < SourceVersion.low then return true
	elseif FinalVersion.low > SourceVersion.low then return false
	elseif FinalVersion.fix < SourceVersion.fix then return true
	elseif FinalVersion.fix > SourceVersion.fix then return false
	else return false -- same version! let return false to avoid unnecessary actions
	end
end


-- Special frames -----------------------------------------------------------------------------------

function MTools.GetRectangleFrame( name, parent, width, height, thikness, color )

	local RectFrame = UI.CreateFrame("Frame", name.."Up", parent)
	
	RectFrame.Down = UI.CreateFrame("Frame", name.."Down", RectFrame)
	RectFrame.Left = UI.CreateFrame("Frame", name.."Left", RectFrame)
	RectFrame.Right = UI.CreateFrame("Frame", name.."Right", RectFrame)

	RectFrame.Down:SetPoint("TOPLEFT", RectFrame, "TOPLEFT", 0, height - thikness)
	RectFrame.Left:SetPoint("TOPLEFT", RectFrame, "TOPLEFT", width - thikness, thikness)
	RectFrame.Right:SetPoint("TOPLEFT", RectFrame, "TOPLEFT", 0, thikness)
	
	RectFrame:SetWidth(width)
	RectFrame.Down:SetWidth(width)
	RectFrame.Left:SetWidth(thikness)
	RectFrame.Right:SetWidth(thikness)

	RectFrame:SetHeight(thikness)
	RectFrame.Down:SetHeight(thikness)
	RectFrame.Left:SetHeight(height - thikness * 2)
	RectFrame.Right:SetHeight(height - thikness * 2)

	RectFrame.Thikness = thikness	
	
	if type(color) == "table" then
		RectFrame:SetBackgroundColor( color.r, color.g, color.b, color.a )
		RectFrame.Down:SetBackgroundColor( color.r, color.g, color.b, color.a )
		RectFrame.Left:SetBackgroundColor( color.r, color.g, color.b, color.a )
		RectFrame.Right:SetBackgroundColor( color.r, color.g, color.b, color.a )	
	end
	
	function RectFrame:SetRectWidth(width)
		self:SetWidth(width)
		self.Down:SetWidth(width)
		Mtools.ClearPoints(self.Left)
		self.Left:SetPoint("TOPLEFT", self, "TOPLEFT", width - self.Thikness, self.Thikness)
	end

	function RectFrame:SetRectHeight(height)
		self:SetHeight(height - self.Thikness * 2)
		self:SetHeight(height - self.Thikness * 2)	
		Mtools.ClearPoints(self.Down)
		self.Down:SetPoint("TOPLEFT", self, "TOPLEFT", 0, height - self.Thikness)	
	end

	return RectFrame
end



-------------------------------------------------------------------------------------------------

-- Clear Frame points / credits goes to UIX authors
-- Helper function for workaround for bug in Frame:ClearPoint
local function PointToName(x, y)
	if x == nil and y == 0.5 then
		return "YCENTER"
	elseif x == 0.5 and y == nil then
		return "XCENTER"
	else
		local result = ""
		
		if y == 0 then
			result = "TOP"
		elseif y == 0.5 then
			result = "CENTER"
		elseif y == 1 then
			result = "BOTTOM"
		end
		
		if x == 0 then
			result = result .. "LEFT"
		elseif x == 0.5 then
			result = result .. "CENTER"
		elseif x == 1 then
			result = result .. "RIGHT"
		end
		
		return result
	end
end

function MTools.ClearPoint(frame, x, y)
	frame:ClearPoint(PointToName(x, y))
end

function MTools.ClearPoints(frame)
	local points = frame:ReadAll()
	local point = { x = nil, y = nil }
	for pos, _ in pairs(points.x) do
		if pos ~= "size" then
			MTools.ClearPoint(frame, tonumber(pos), nil)
		end
	end
	for pos, _ in pairs(points.y) do
		if pos ~= "size" then
			MTools.ClearPoint(frame, nil, tonumber(pos))
		end
	end
end

-------------------------------------------------------------------------------------------------

function MTools.buildWindowResizer(name, parent, IconSource, IconPath, IconSize, BlockStep, MinWidth, MinHeight, MaxWidth, MaxHeight)
	
	if BlockStep and MinHeight and (MinHeight < IconSize) then MinHeight = IconSize * MinHeight end
	IconSize = IconSize or 24

	local resizeIcon = UI.CreateFrame("Texture", name, parent)
	resizeIcon:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT")
	resizeIcon:SetHeight(IconSize)
	resizeIcon:SetWidth(IconSize)	
	resizeIcon:SetTexture(IconSource, IconPath)

	MTools.makeResizable(resizeIcon, parent, BlockStep, MinWidth, MinHeight, MaxWidth, MaxHeight)

	return resizeIcon
end


function MTools.makeResizable(resizeIcon, object, blockStep, minWidth, minHeight, maxWidth, maxHeight)
	
	resizeIcon.ResizeEnabled = true
	resizeIcon.BlockStep = blockStep or false
	resizeIcon.MinWidth = minWidth or 16
	resizeIcon.MinHeight = minHeight or 16
	resizeIcon.MaxWidth = maxWidth
	resizeIcon.MaxHeight = maxHeight
	
	function resizeIcon.Event:LeftDown()
		self.MouseDown = self.ResizeEnabled
		mouseData = Inspect.Mouse()
		self.MyStartWidth = object:GetWidth()
		self.MyStartHeight = object:GetHeight()
		self.StartX = mouseData.x -- self.MyStartX
		self.StartY = mouseData.y -- self.MyStartY
	end
	
	function resizeIcon.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			local dx = self.MyStartWidth + mouseX - self.StartX
			local dy = self.MyStartHeight + mouseY - self.StartY
			if self.BlockStep then
				local d = self:GetWidth()
				dy = math.floor((dy + d/2)/d)* d
			end
			if dx < self.MinWidth then dx = self.MinWidth
			elseif self.MaxWidth and (dx > self.MaxWidth) then dx = self.MaxWidth end
			if dy < self.MinHeight then dy = self.MinHeight
			elseif self.MaxHeight and (dy > self.MaxHeight) then dy = self.MaxHeight end
			
			object:SetWidth(dx)
			object:SetHeight(dy)
			if object.AdjustToSize then object:AdjustToSize() end
		end
	end
	
	function resizeIcon.Event:LeftUpoutside()
		self.MouseDown = false
	end
	
	function resizeIcon.Event:LeftUp()
		self.MouseDown = false
	end
end


-------------------------------------------------------------------------------------------------

-- Make a windows moveable / credits goes to Karuul authors
function MTools.buildWindowMover(name, parent, mHeight, mWidthMinus)

	mHeight = mHeight or 50
	mWidthMinus = mWidthMinus or 40
	
	local windowMover = UI.CreateFrame("Frame", name, parent)
	windowMover:SetPoint("TOPLEFT", parent, "TOPLEFT")
	windowMover:SetHeight(mHeight)
	windowMover:SetWidth(parent:GetWidth() - mWidthMinus)
	windowMover:SetBackgroundColor(0, 0, 0, 0)

	MTools.makeMovable(windowMover, parent)

	return windowMover
end


function MTools.makeMovable(object, movingObject)
	if movingObject == nil then
		movingObject = object
	end
	
	object.movable = true
	
	function object.Event:LeftDown()
		self.MouseDown = self.movable
		mouseData = Inspect.Mouse()
		self.MyStartX = movingObject:GetLeft()
		self.MyStartY = movingObject:GetTop()
		self.StartX = mouseData.x - self.MyStartX
		self.StartY = mouseData.y - self.MyStartY
	end
	
	function object.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			MTools.ClearPoints(movingObject)
			movingObject:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (mouseX - self.StartX), (mouseY - self.StartY))
		end
	end
	
	function object.Event:LeftUp()
		self.MouseDown = false
	end
end

function MTools.buildButton(name, parent, width, height, label, skin)

	local button = UI.CreateFrame("RiftButton", name, parent)
	if skin == "close" then
		button:SetSkin("close")
		return button
	else
		button:SetText(label)
		button:SetHeight(height)
		button:SetWidth(width)
	end
	
	return button
	
end

function MTools.buildLabel(name, parent, width, height, text, wrap)

	if wrap == nil then wrap = false end
	local label = UI.CreateFrame("Text", name, parent)
	label:SetWordwrap(wrap)
	label:SetText(text)
	label:SetFontSize(height)
	label:SetWidth(width)
	label:SetFontColor(1, 1, 1, 1)
	
	function label.setWrapText(text, html)
		label:SetWordwrap(true)
		label:SetText(text, html or false)
	end
	
	return label
	
end

function MTools.buildCheckBox(name, parent, width, height, label, reverse)

	local checkBox = UI.CreateFrame("RiftCheckbox", name, parent)
	checkBox:SetHeight(height)
	checkBox:SetWidth(height)
	checkBox.label = MTools.buildLabel(name .. "Label", checkBox, width - height - 3, height-5, label)
	if reverse then
		checkBox.label:SetPoint("CENTERRIGHT", checkBox, "CENTERLEFT", 3, 0)
	else
		checkBox.label:SetPoint("CENTERLEFT", checkBox, "CENTERRIGHT", 3, 0)
	end
	
	return checkBox
	
end

function MTools.buildEditBox(name, parent, width, height, label, boxWidth)

	local editBox = UI.CreateFrame("Frame", name, parent)
	editBox:SetPoint("TOPLEFT", parent, "TOPLEFT")
	editBox:SetHeight(height)
	editBox:SetWidth(width)
	editBox.label = MTools.buildLabel(name .. "Label", editBox, width, height - 5, label)
	editBox.label:ClearWidth()
	editBox.label:SetPoint("CENTERLEFT", editBox, "CENTERLEFT")
	editBox.label:SetLayer(1)
	editBox.text = UI.CreateFrame("RiftTextfield", name .. "Text", editBox)
	
	if boxWidth == nil then
		editBox.text:SetPoint("CENTERLEFT", editBox.label, "CENTERRIGHT")
		editBox.text:SetHeight(height)
		editBox.text:SetWidth(width - editBox.label:GetWidth())
	else
		editBox.text:SetPoint("CENTERY", editBox.label, "CENTERY")
		editBox.text:SetPoint("RIGHT", editBox, "RIGHT")
		editBox.text:SetHeight(height)
		editBox.text:SetWidth(boxWidth)
	end
		
	editBox.text:SetBackgroundColor(0, 0, 0)
	editBox.text:SetLayer(1)
	
	function editBox.setLabel(label)
		editBox.label:SetText(label)
		editBox.text:SetWidth(editBox:GetWidth() - editBox.label:GetWidth())		
	end
	
	return editBox
end

-------------------------------------------------------------------------------------------------

-- More interface Objects!

function MTools.buildShadowTitle(name, parent, width, height, label, BackColor, TitleColor)

	local title = UI.CreateFrame("Frame", name, parent)
	title:SetHeight(height)
	title:SetWidth(width)
	title.label = MTools.buildLabel(name .. "Label", title, width - 10, height - 5, label)
	title.label:SetPoint("TOPLEFT", title, "TOPLEFT", 10, 0)
	
	if TitleColor then title.label:SetFontColor(unpack(TitleColor)) end
	if BackColor then title:SetBackgroundColor(unpack(BackColor))
		else title:SetBackgroundColor(0, 0, 0, 0.7) end
	
	return title	
end


local function buildDefaultSlider(name, parent, width, height, label, value, minV, maxV)

	local slider = UI.CreateFrame("RiftSlider", name, parent)
	
	slider:SetRange(minV, maxV)
	slider:SetHeight(height)
	slider:SetWidth(width)
	slider.label = MTools.buildLabel(name .. "Label", slider, width - height * 3, height - 4, label)
	slider.label:SetPoint("TOPLEFT", slider, "TOPLEFT", 0, -height-6)
	
	slider.Value = UI.CreateFrame("Text", name .. "Value", slider)
	slider.Value:SetFontSize(height-4)
	slider.Value:SetFontColor(1, 1, 1, 1)
	slider.Value:SetPoint("TOPRIGHT", slider, "TOPRIGHT", 0, -height-6)

	function slider:GetValue()
		return self:GetPosition()
	end
	
	function slider.Event:SliderChange()
		self:SetValue(self:GetPosition())
	end
	
	return slider	
	
end

function MTools.buildIntegerSlider(name, parent, width, height, label, value, minV, maxV, unit, decimal)

	local d = 10^(decimal or 0)
	local slider = buildDefaultSlider(name, parent, width, height, label, value/d, minV/d, maxV/d)
	slider.vUnit = unit or ""
	slider.decimal = d
	
	function slider:SetValue(v)
		self:SetPosition(v)
		local s = tostring(v*self.decimal)..self.vUnit
		self.Value:SetText(s)		
	end
	
	function slider:SetDecimalValue(v)
		self:SetValue(math.floor(v/self.decimal))
	end
	
	function slider:GetDecimalValue()
		return (self:GetPosition()*self.decimal)
	end
	
	slider:SetValue(value)
	
	return slider
end

function MTools.buildRealSlider(name, parent, width, height, label, value, minV, maxV, digit, unit)

	local x = 10^digit
	local slider = buildDefaultSlider(name, parent, width, height, label, value, math.floor(minV*x), math.floor(maxV*x))
	slider.Digit = x
	slider.vUnit = ""
	if unit then slider.vUnit = unit end

	function slider:GetRealValue()
		return self:GetPosition()/slider.Digit
	end
	
	function slider:SetRealValue(v)
		return self:SetValue(math.floor(v*self.Digit))
	end
	
	function slider:SetValue(v)
		self:SetPosition(v)
		local s = tostring(math.floor(v/self.Digit)) .. "." .. tostring(v % self.Digit)..self.vUnit
		self.Value:SetText(s)		
	end
	
	slider:SetRealValue(value)
	
	return slider
end

function MTools.buildTextSlider(name, parent, width, height, label, value, Text, Helper)

	if type(Text) ~= "table" then return nil end

	local slider = buildDefaultSlider(name, parent, width, height, label, value, 1, #Text)
	
	slider.textTable = MTools.CopyTable(Text)
	if type(Helper) == "table" then 
		slider.helperTable = MTools.CopyTable(Helper)
	
		slider.helper = UI.CreateFrame("Text", name .. "Value", slider)
		slider.helper:SetFontSize(height-7)
		slider.helper:SetFontColor(1, 1, 1, 1)
		slider.helper:SetPoint("BOTTOMLEFT", slider, "BOTTOMLEFT", 0, 10)
	end

	function slider:SetValue(v)
		self:SetPosition(v)
		self.Value:SetText(self.textTable[v])
		if self.helperTable then self.helper:SetText(self.helperTable[v]) end
	end
	
	slider:SetValue(value)
	
	return slider
end

function MTools.buildText(name, parent, text, size, color, shadow)

	local label = UI.CreateFrame("Text", name, parent)
	if color then label:SetFontColor(unpack(color)) end
	if size then label:SetFontSize(size) end
	label:SetText(text)
	if shadow then
		label.labelShadow = UI.CreateFrame("Text", name, parent)
		label.labelShadow:SetLayer(label:GetLayer()-1)
		label.labelShadow:SetFontColor(0, 0, 0, 0)
		label.labelShadow:SetText(text)
		label.labelShadow:SetPoint("TOPLEFT", label, "TOPLEFT", 2, 2)
		if size then label.labelShadow:SetFontSize(size) end
	end

	return label	
end

function MTools.buildRadioGroup(name, parent, width, height, rLabel, value, elementText, DX)

	local function buildRadioElement(Name, Parent, Width, Height, Text, index)
		local element = MTools.buildCheckBox(name, Parent, Width, Height, Text)
		
		element.Index = index
		
		function element.Event:CheckboxChange()
			local eParent = self:GetParent()
			if not eParent.updating then eParent:ElementClicked(self.Index) end
		end
	
		return element
	end

	local label = UI.CreateFrame("Text", name, parent)
	label:SetText(rLabel)
	label:SetWidth(width)
	label:SetHeight(height)
	label:SetFontSize(height - 4)
	
	label.dx = DX or 0
	label.element = {}
	for i = 1, #elementText do 
		label.element[i] = buildRadioElement(name..tostring(i), label, width, height, elementText[i], i)
		label.element[i]:SetPoint("TOPLEFT", label, "TOPLEFT", height-label.dx, 4+(height+2)*i)
	end
	label.value = value
	label.updating = false
	
	function label:ElementClicked(I)
		self.updating = true
		for i = 1, #self.element do 
			if i == I then 
				self.element[i]:SetChecked(true) 
			else
				self.element[i]:SetChecked(false) 
			end
		end
		self.updating = false
		self.value = I
	end
	
	function label:SetValue(v)
		self:ElementClicked(v)
	end
	
	function label:GetValue()
		return self.value
	end

	label:SetValue(value)
	
	return label	
end


function MTools.buildIconButton(parent, IconPath)

	local Button = UI.CreateFrame("Texture", "HermesIconButton", parent)
	
	Button.IconPath = IconPath
	Button.LeftPressed = false
	Button.Moving = false
	Button.MouseIsIn = false
	
	function Button:SetButtonUp()
		self:SetTexture(RWData.AddonIdentifier, self.IconPath[1])
	end
	
	function Button:SetButtonDown()
		self:SetTexture(RWData.AddonIdentifier, self.IconPath[3])
	end
	
	function Button:SetButtonMouseIn()
		self:SetTexture(RWData.AddonIdentifier, self.IconPath[2])
	end
		
	-- Handlers ----
	
	function Button.Event:MouseIn()
		self.MouseIsIn = true
		if self.LeftPressed and (not self.Moving) then
			self:SetButtonDown()
		else
			self:SetButtonMouseIn()
		end
	end
	
	function Button.Event:MouseOut()
		self.MouseIsIn = false
		self:SetButtonUp()
	end
	
	function Button.Event:LeftDown()
		self.LeftPressed = true
		self:SetButtonDown()
	end	
	
	function Button.Event:LeftUp()
		if self.LeftPressed then
			self.LeftPressed = false
			self:SetButtonMouseIn()
			if self.LeftUpHandler then self:LeftUpHandler() end
		end
	end	
	
	function Button.Event:LeftUpoutside()
		self.LeftPressed = false
	end

	Button:SetButtonUp()
	return Button
end


-- General Windows ----------------------------------------

local function buildYesNoWindow()

	local window = UI.CreateFrame("RiftWindow", "MToolsYesNoWindow", MTools.GetContext())
	window:SetVisible(false)
	
	window:SetPoint("CENTER", UIParent, "CENTER")
	
	window.Line1 = UI.CreateFrame("Text", "YesNoLine1", window)
	window.Line1:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 50)
	window.Line2 = UI.CreateFrame("Text", "YesNoLine1", window)
	window.Line2:SetPoint("TOPCENTER", window.Line1, "TOPCENTER", 0, 20)
	
	window.btYes =  MTools.buildButton("btYes", window, 100, 30, "Oui")
	window.btYes:SetPoint("BOTTOMCENTER", window, "BOTTOMCENTER", -60, -15)

	window.btNo =  MTools.buildButton("btNo", window, 100, 30, "Non")
	window.btNo:SetPoint("BOTTOMCENTER", window, "BOTTOMCENTER", 60, -15)	
	
	window.mover = MTools.buildWindowMover("WindowMover", window)
	
	window.CallBackFunction = nil
	
	function window:InitWindow(callBackFunc, Text1, Text2, Title, width, height, fontSize, YesText, NoText)
		local h = 140
		window.CallBackFunction = callBackFunc
		window:SetTitle(Title)
		if YesText then window.btYes:SetText(YesText) end
		if NoText then window.btNo:SetText(NoText) end
		if fontSize then window.Line1:SetFontSize(fontSize) end
		if fontSize then window.Line2:SetFontSize(fontSize) end
		if Text1 then window.Line1:SetText(Text1) window.Line1:SetVisible(true) else window.Line1:SetText("") window.Line1:SetVisible(false) end
		if Text2 then h=h+20 window.Line2:SetText(Text2) window.Line2:SetVisible(true) else window.Line2:SetText("") window.Line2:SetVisible(false) end
		
		if not width then width = 0 end
		width = math.max(width, window.Line1:GetWidth()+30, window.Line2:GetWidth()+30, 260)
		window:SetWidth(width)
		window.mover:SetWidth(width)
		
		if not height then height = 0 end
		height = math.max(height, h)
		window:SetHeight(height)
	end
	
	function window:ChoiceMade(choice)
		window:SetVisible(false)
		if window.CallBackFunction then window.CallBackFunction(choice) end
	end
	
	function window.btNo.Event:LeftPress()
		window:ChoiceMade(false)
	end
	
	function window.btYes.Event:LeftPress()
		window:ChoiceMade(true)
	end
	
	window:InitWindow(nil, nil, nil, "Confirmation", 250, 200, 20)
	MTools.YesNoWindow = window
end

function MTools.ShowYesNoWindow(callBackFunc, Text1, Text2, Title, width, height, fontSize, YesText, NoText)
	if not MTools.YesNoWindow then buildYesNoWindow() end
	MTools.YesNoWindow:InitWindow(callBackFunc, Text1, Text2, Title, width, height, fontSize, YesText, NoText)
	MTools.YesNoWindow:SetVisible(true)
end


-- launching ! -----------------------------------------------------------


fastCosInit()