---------------------------------------------------
--                                               --
--              Hermes Addon Button              --
--                                               --
--    Access more easily to Hermes functions!    --
--                                               --
---------------------------------------------------


RWConfig.HermesButton = {
	
	Active = true,
	
	Location = { x = 400, y = 200 },
	
	
	OpenAllMenu = true,
	CloseOnClick = true,
	OpenOnMouseOver = true,
	Locked = false,
	useSmallicon = false,
	
	showHelp = true,
	
	ButtonOrder = {}
}

RWConst.HermesButton = {

	IconSeparation = 4,
	AnimTime = 0.2,
	FontSize = 18,
	MenuTTL = 1,
	FadingTime = 2,
	MenuOpenTTL = 2,
	
	InfoBackColor = {0, 0, 0, 0.5},
	InfoFontSize = 13,
	InfoHeight = 40,
	InfoY0 = 2,
	InfoX0 = 4,
	InfoDX = 10,
	InfoDY = 5,
	
	MainBtnPath = {
		"Resources\\Button\\btnHermesUp.png",
		"Resources\\Button\\btnHermesIn.png",
		"Resources\\Button\\btnHermesDown.png"	
	}

}

RWData.HermesButton = {


}


RWRoot.HermesButton = {

	MainButton = nil,
	InfoFrame = nil,
	
	IconSize = 0,

	AnimCounter = 0,
	ModMenuOpen = 0,
	NeedUpdate = false,
	MenuTimeLimit = 0,
	
	ModTitleX0 = 0,
	ModTitleY0 = 3,
	ModTitleAllign = "",

	dx = -1,
	dy = 1
}


-- General functions -------------------------------------------------------

local function StartAnimation()
	RWRoot.HermesButton.AnimCounter = RWRoot.HermesButton.AnimCounter + 1
end 
	
local function DoneAnimation()
	RWRoot.HermesButton.AnimCounter = RWRoot.HermesButton.AnimCounter - 1
end 
	
local function StartModMenuAnim()
	RWRoot.HermesButton.AnimCounter = RWRoot.HermesButton.AnimCounter + 1
	RWRoot.HermesButton.ModMenuOpen = RWRoot.HermesButton.ModMenuOpen + 1
end 
	
local function DoneModMenuAnim()
	RWRoot.HermesButton.AnimCounter = RWRoot.HermesButton.AnimCounter - 1
	RWRoot.HermesButton.ModMenuOpen = RWRoot.HermesButton.ModMenuOpen - 1
	RWRoot.HermesButton.MenuTimeLimit = RWRoot.CurrentTime + RWConst.HermesButton.MenuOpenTTL
end 
	

local function InitData()
	
	if RWConfig.HermesButton.useSmallicon then
		RWRoot.HermesButton.ModTitleY0 = -4
		RWRoot.HermesButton.IconSize = 32
	else
		RWRoot.HermesButton.ModTitleY0 = 3
		RWRoot.HermesButton.IconSize = 48
	end

	RWRoot.HermesButton.dx = RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation
	RWRoot.HermesButton.dy = RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation
	RWRoot.HermesButton.ModTitleX0 = 0
	RWRoot.HermesButton.ModTitleAllign = "CENTERLEFT"

	if RWConfig.HermesButton.Location.x > RWConst.MiddleScreen.x then
		RWRoot.HermesButton.dx = -RWRoot.HermesButton.dx
		RWRoot.HermesButton.ModTitleX0 = RWRoot.HermesButton.IconSize
		RWRoot.HermesButton.ModTitleAllign = "CENTERRIGHT"
	end
	if RWConfig.HermesButton.Location.y > RWConst.MiddleScreen.y then
		RWRoot.HermesButton.dy = -RWRoot.HermesButton.dy
	end		
end


-- InfoFrame -----------------------------------------------------------

local function buildInfoFrame(parent)

	local InfoFrame = UI.CreateFrame("Frame", "InfoFrame", parent)
	InfoFrame:SetVisible(false)
	
	InfoFrame:SetHeight(RWConst.HermesButton.InfoHeight)
	InfoFrame:SetBackgroundColor(unpack(RWConst.HermesButton.InfoBackColor))
	
	InfoFrame.Text1 = UI.CreateFrame("Text", "InfoFrame", InfoFrame)
	InfoFrame.Text2	= UI.CreateFrame("Text", "InfoFrame", InfoFrame)
	
	InfoFrame.Text1:SetFontSize(RWConst.HermesButton.InfoFontSize)
	InfoFrame.Text2:SetFontSize(RWConst.HermesButton.InfoFontSize)
	InfoFrame.Text1:SetPoint("TOPLEFT", InfoFrame,"TOPLEFT", RWConst.HermesButton.InfoX0, RWConst.HermesButton.InfoY0-1)
	InfoFrame.Text2:SetPoint("BOTTOMLEFT", InfoFrame,"BOTTOMLEFT", RWConst.HermesButton.InfoX0, -RWConst.HermesButton.InfoY0)
	
	InfoFrame.ShownButton = nil
	
	function InfoFrame:ShowButtonInfo(button)
		if RWConfig.HermesButton.showHelp then
			self.ShownButton = button
			self.Text1:SetText(button.Description)
			self.Text2:SetText("/hermes "..button.command)
			self:SetWidth(math.max(self.Text1:GetWidth(), self.Text2:GetWidth())+2*RWConst.HermesButton.InfoX0)	
			self:SetVisible(true)
		end
	end

	function InfoFrame:HideInfo(sender)
		if self.ShownButton == sender then
			self:SetVisible(false)
		end
	end

	function InfoFrame:AdjustPosition()
		local SelfAllign = ""
		local TargetAllign = ""
		local dx = 0
		local dy = 0
		if RWConfig.HermesButton.Location.x > RWConst.MiddleScreen.x then
			SelfAllign = "RIGHT"
			TargetAllign = "LEFT"
			dx = -RWConst.HermesButton.InfoDX
		else
			SelfAllign = "LEFT"
			TargetAllign = "RIGHT"
			dx = RWConst.HermesButton.InfoDX
		end
		if RWConfig.HermesButton.Location.y > RWConst.MiddleScreen.y then
			SelfAllign = "TOP"..SelfAllign
			TargetAllign = "TOP"..TargetAllign		
			dy = RWConst.HermesButton.InfoDY
		else
			SelfAllign = "BOTTOM"..SelfAllign
			TargetAllign = "BOTTOM"..TargetAllign
			dy = -RWConst.HermesButton.InfoDY
		end
		MTools.ClearPoints(self)
		self:SetPoint(SelfAllign, self:GetParent(), TargetAllign, dx, dy)
	end

	return InfoFrame
end


-- Addon Button -----------------------------------------------------------

local function buildAddonButton(parent, command, Description, IconPath)

	local Button = MTools.buildIconButton(parent, IconPath)
	
	Button:SetHeight(RWRoot.HermesButton.IconSize)
	Button:SetWidth(RWRoot.HermesButton.IconSize)
	
	Button.command = command
	Button.Description = Description
	
	Button.AnimLimit = 0
	Button.AnimLimitGlobal = 0

	return Button
end

local function buildButtonElement(parent, command, Description, IconPath)

	local ButtonElement = buildAddonButton(parent, command, Description, IconPath)

	function ButtonElement.Event:MouseIn()
		RWRoot.HermesButton.InfoFrame:ShowButtonInfo(self)
		if self.LeftPressed and (not self.Moving) then
			self:SetButtonDown()
		else
			self:SetButtonMouseIn()
		end
		self:GetParent():GetParent():MouseInside()
	end
	
	function ButtonElement.Event:MouseOut()
		RWRoot.HermesButton.InfoFrame:HideInfo(self)
		self:SetButtonUp()
		self:GetParent():GetParent():MouseOutside()
	end	
	
	function ButtonElement.Event:LeftUp()
		if self.LeftPressed then
			self.LeftPressed = false
			self:SetButtonMouseIn()
			if RWConfig.HermesButton.CloseOnClick then
				self:GetParent():GetParent():GetParent():GetParent():CloseMenu()
			end
			RWRoot.CommandLine("", self.command)
		end
	end
	
	return ButtonElement
end


-- Mod Button ------------------------------------------------

local function buildModButton(parent, ModName, command, Description, IconPath)

	local ModButton = buildAddonButton(parent, command, Description, IconPath)
		
	ModButton.ElementParent = UI.CreateFrame("Frame", "MenuParent", ModButton)
	ModButton.ElementParent:SetVisible(false)
	
	ModButton.Title = UI.CreateFrame("Text", "MenuParent", ModButton.ElementParent)
	ModButton.Title:SetFontSize(RWConst.HermesButton.FontSize)
	ModButton.Title:SetText(ModName)
	ModButton.TitleSize = ModButton.Title:GetWidth()
	
	ModButton.Element = {}
	ModButton.MaxElement = 0
	
	ModButton.MenuMaxDistance = 0

	ModButton.AnimPhase = 0
	ModButton.AnimStartTime = 0
	ModButton.MenuOpen = false
	ModButton.Animating = false
		
	
	-- Animation ----
	
	function ModButton:AnimButton(r)
		local x = 0
		for i = 1, self.MaxElement do
			--x = x + RWRoot.HermesButton.dx
			x = x + RWRoot.HermesButton.dx
			if r < self.Element[i].AnimLimit then x = (x * r) / self.Element[i].AnimLimit end
			--MTools.ClearPoints(self.Element[i])
			self.Element[i]:ClearPoint("TOPLEFT")
			self.Element[i]:SetPoint("TOPLEFT", self.ElementParent, "TOPLEFT", x, 0)
		end
		local a = r / RWConst.HermesButton.AnimTime
		x = (x + RWRoot.HermesButton.dx + RWRoot.HermesButton.ModTitleX0) * a
	
		MTools.ClearPoints(self.Title)
		--self.Title:ClearPoint(RWRoot.HermesButton.ModTitleAllign)
		self.Title:SetPoint(RWRoot.HermesButton.ModTitleAllign, self.ElementParent, "CENTERLEFT", x, RWRoot.HermesButton.ModTitleY0)
		self.Title:SetWidth(a*self.TitleSize)	
	end
	
	function ModButton:FixButtonPosition()
		local x = 0
		for i = 1, self.MaxElement do
			x = x + RWRoot.HermesButton.dx
			--MTools.ClearPoints(self.Element[i])
			self.Element[i]:ClearPoint("TOPLEFT")
			self.Element[i]:SetPoint("TOPLEFT", self.ElementParent, "TOPLEFT", x, 0)
		end
		self:InitPosition()
		self.Title:SetWidth(self.TitleSize)	
	end
	
	function ModButton:OpenMenu()
		if not self.Animating then StartModMenuAnim() end
		self.AnimPhase = 1
		self.Animating = true
		self.MenuOpen = true
		self.AnimStartTime = RWRoot.CurrentTime
		self:AnimButton(0)		
		self.ElementParent:SetAlpha(1)	
		self.ElementParent:SetVisible(true)			
	end

	function ModButton:CloseMenu()
		if not self.Animating then StartAnimation() end
		self.AnimPhase = 1
		self.Animating = true
		self.MenuOpen = false
		self.AnimStartTime = RWRoot.CurrentTime
		self:AnimButton(RWConst.HermesButton.AnimTime)
	end

	function ModButton:startAnimP3()
		StartAnimation()
		self.Animating = true
		self.AnimPhase = 3
		self.AnimStartTime = RWRoot.CurrentTime + RWConst.HermesButton.MenuTTL
	end

	function ModButton:startAnimP2()
		self.AnimPhase = 2
		self.Animating = false
		DoneAnimation()
	end
	
	function ModButton:Update()
		if self.AnimPhase == 1 then
			if self.MenuOpen then 
				local r = RWRoot.CurrentTime - self.AnimStartTime
				if r < RWConst.HermesButton.AnimTime then 
					self:AnimButton(r)
				else
					self:FixButtonPosition()
					self:startAnimP2()
					if not self.MouseIsIn then self:startAnimP3() end
				end
			else			
				local r = RWConst.HermesButton.AnimTime - RWRoot.CurrentTime + self.AnimStartTime
				if r >= 0 then 
					self:AnimButton(r)
				else
					self.Animating = false
					self.AnimPhase = 0
					self.ElementParent:SetVisible(false)
					DoneModMenuAnim()
					RWRoot.HermesButton.InfoFrame:SetVisible(false)
				end				
			end	
		elseif self.AnimPhase == 3 then
			if RWRoot.CurrentTime >= self.AnimStartTime then
				self.AnimPhase = 4
				self.AnimStartTime = RWRoot.CurrentTime + RWConst.HermesButton.FadingTime			
			end		
		elseif RWRoot.CurrentTime < self.AnimStartTime then
			self.ElementParent:SetAlpha((self.AnimStartTime - RWRoot.CurrentTime) / RWConst.HermesButton.FadingTime)
		
		else
			self.ElementParent:SetVisible(false)
			self.Animating = false
			self.MenuOpen = false
			self.AnimPhase = 0		
			DoneModMenuAnim()		
		end
	end
	
	
	-- Handlers ----
	
	function ModButton:MouseInside()
		self.MouseIsIn = true
		if (self.AnimPhase == 3) or (self.AnimPhase == 4) then
			self.ElementParent:SetAlpha(1)
			self:startAnimP2()			
		end
	end
	
	function ModButton:MouseOutside()
--		RWRoot.HermesButton.MenuTimeLimit = RWRoot.CurrentTime + RWConst.HermesButton.MenuOpenTTL
		self.MouseIsIn = false
		if self.AnimPhase == 2 then self:startAnimP3() end	
	end
	
	function ModButton.Event:MouseIn()
		RWRoot.HermesButton.InfoFrame:ShowButtonInfo(self)
		self.MouseIsIn = true
		if self.AnimPhase == 0 then 
			if not self.MenuOpen then
				self:OpenMenu()			
			end
		else
			self:MouseInside()
		end
		
		if self.LeftPressed and (not self.Moving) then
			self:SetButtonDown()
		else
			self:SetButtonMouseIn()
		end
	end
	
	function ModButton.Event:MouseOut()
		RWRoot.HermesButton.InfoFrame:HideInfo(self)
		self:SetButtonUp()
		self:MouseOutside()
	end
	
	function ModButton.Event:LeftUp()
		if self.LeftPressed then
			self.LeftPressed = false
			self:SetButtonMouseIn()
			self:CloseMenu()
			if RWConfig.HermesButton.CloseOnClick then
				self:GetParent():GetParent():CloseMenu()
			end
			RWRoot.CommandLine("", self.command)
		end
	end	
	
	-- Init ----
	
	function ModButton:RegisterModButton(command, Description, IconPath)
		self.MaxElement = self.MaxElement + 1
		self.Element[self.MaxElement] = buildButtonElement(self.ElementParent, command, Description, IconPath)
		return self
	end
	
	function ModButton:UpdateModAnimLimit(Dist, MaxDist)
		self.AnimLimitGlobal = Dist * RWConst.HermesButton.AnimTime / MaxDist
		Dist = Dist^2
		for i = 1, self.MaxElement do
			self.Element[i].AnimLimitGlobal = math.sqrt((i*(RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation))^2 + Dist) * RWConst.HermesButton.AnimTime / MaxDist
		end	
	end
	
	function ModButton:InitPosition()
		MTools.ClearPoints(self.Title)
		--self.Title:ClearPoint(RWRoot.HermesButton.ModTitleAllign)
		self.Title:SetPoint( RWRoot.HermesButton.ModTitleAllign, self.ElementParent, "CENTERLEFT", 
			((self.MaxElement + 1) * RWRoot.HermesButton.dx) + RWRoot.HermesButton.ModTitleX0, RWRoot.HermesButton.ModTitleY0)
	end
	
	function ModButton:UpdateChanges()
		self:ClearPoint("TOPLEFT")
		self:SetPoint("TOPLEFT", self.ElementParent, "TOPLEFT")
		self:SetHeight(RWRoot.HermesButton.IconSize)
		self:SetWidth(RWRoot.HermesButton.IconSize)
		self.MenuMaxDistance = self.MaxElement * (RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation)
		for i = 1, self.MaxElement do
			self.Element[i]:ClearPoint("TOPLEFT")
			self.Element[i]:SetPoint("TOPLEFT", self.ElementParent, "TOPLEFT", i*RWRoot.HermesButton.dx, 0)
			self.Element[i]:SetHeight(RWRoot.HermesButton.IconSize)
			self.Element[i]:SetWidth(RWRoot.HermesButton.IconSize)
			self.Element[i].AnimLimit = i * (RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation) * RWConst.HermesButton.AnimTime / self.MenuMaxDistance
		end
	end
	
	ModButton:InitPosition()
	
	return ModButton
end


-- Main Button ------------------------------------------------

local function buildMainButton()

	local MainButton = buildAddonButton(RWRoot.context, "", "", RWConst.HermesButton.MainBtnPath)
	
	MainButton.ElementParent = UI.CreateFrame("Frame", "MenuParent", MainButton)
	MainButton.ElementParent:SetPoint("TOPLEFT", MainButton, "TOPLEFT")
	MainButton.ElementParent:SetVisible(false)
	
	MainButton.Element = {}
	MainButton.MaxElement = 0
	
	MainButton.MouseX0 = 0
	MainButton.MouseY0 = 0
	
	MainButton.MaxDistance = 0
	MainButton.MenuMaxDistance = 0
	
	MainButton.AnimStartTime = 0
	MainButton.Animating = false
	MainButton.MenuOpen = false
	MainButton.MenuWasOpen = false
		
	
	-- Functions ----
	
	function MainButton:RegisterMenuButton(ModName, command, Description, IconPath)
		self.MaxElement = self.MaxElement + 1
		self.Element[self.MaxElement] = buildModButton(self.ElementParent, ModName, command, Description, IconPath)
		
		return self.Element[self.MaxElement]
	end	
	
	
	-- Animation ----
	
	function MainButton:AnimButton(r)
		local x = 0
		for i = 1, self.MaxElement do
			x = x + RWRoot.HermesButton.dy
			if r < self.Element[i].AnimLimit then x = (x * r) / self.Element[i].AnimLimit end
			self.Element[i].ElementParent:ClearPoint("TOPLEFT")
			self.Element[i].ElementParent:SetPoint("TOPLEFT", self, "TOPLEFT", 0, x)
			--self.Element[i]:AnimButton(r)
		end		
	end
	
	function MainButton:FixButtonPosition()
		local x = 0
		for i = 1, self.MaxElement do
			x = x + RWRoot.HermesButton.dy
			self.Element[i].ElementParent:ClearPoint("TOPLEFT")
			self.Element[i].ElementParent:SetPoint("TOPLEFT", self, "TOPLEFT", 0, x)
			--self.Element[i]:AnimButton(r)
		end		
	end
	
	function MainButton:OpenMenu()
		if not self.Animating then StartAnimation() end
		self.Animating = true
		self.MenuOpen = true
		self.AnimStartTime = RWRoot.CurrentTime
		RWRoot.HermesButton.NeedUpdate = true
		self:AnimButton(0)		
		self.ElementParent:SetVisible(true)	
		if RWConfig.HermesButton.OpenAllMenu then
			for i = 1, self.MaxElement do self.Element[i]:OpenMenu() end
		end
	end

	function MainButton:CloseMenu()
		if not self.Animating then StartAnimation() end
		self.Animating = true
		self.MenuOpen = false
		self.AnimStartTime = RWRoot.CurrentTime
		self:AnimButton(RWConst.HermesButton.AnimTime)		
		for i = 1, self.MaxElement do
			if self.Element[i].MenuOpen then self.Element[i]:CloseMenu() end
		end
	end
	
	function MainButton:Update()
	
		if RWRoot.HermesButton.AnimCounter > 0 then
		
		-- Animation is in progress
		if self.Animating then
			if self.MenuOpen then 
				local r = RWRoot.CurrentTime - self.AnimStartTime
				if r < RWConst.HermesButton.AnimTime then 
					self:AnimButton(r)
				else
					self.Animating = false
					self:FixButtonPosition() 
					DoneAnimation()
				end
			else
				local r = RWConst.HermesButton.AnimTime - RWRoot.CurrentTime + self.AnimStartTime
				if r >= 0 then 
					self:AnimButton(r)
				else
					self.Animating = false
					RWRoot.HermesButton.NeedUpdate = false
					DoneAnimation()
					self.ElementParent:SetVisible(false)
					RWRoot.HermesButton.InfoFrame:SetVisible(false)
				end
			end
		end
		
		for i = 1, self.MaxElement do 
			if self.Element[i].Animating then self.Element[i]:Update() end
		end
		
		-- No animation! checking the TTL
		elseif (RWRoot.HermesButton.ModMenuOpen == 0) and (RWRoot.CurrentTime >= RWRoot.HermesButton.MenuTimeLimit) and (self.MenuOpen) then
			self:CloseMenu()	
		
		end
	end
	
	
	-- Handlers ----
	
	function MainButton.Event:MiddleDown()
		RWConfig.HermesButton.Locked = not RWConfig.HermesButton.Locked
	end
	
	function MainButton.Event:LeftDown()
		local mouseData = Inspect.Mouse()
		self.MouseX0 = mouseData.x - self:GetLeft()
		self.MouseY0 = mouseData.y - self:GetTop()
		self.LeftPressed = true
		self:SetButtonDown()
	end	
	
	function MainButton.Event:MouseMove(mouseX, mouseY)
		if self.LeftPressed and (not RWConfig.HermesButton.Locked) then
			self:ClearPoint("TOPLEFT")
			self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (mouseX - self.MouseX0), (mouseY - self.MouseY0))
			if not self.Moving then 
				self.MenuWasOpen = self.MenuOpen
				self:SetButtonMouseIn()
				self.Moving = true
				if self.MenuOpen then self:CloseMenu() end
			end
		end
	end
	
	function MainButton.Event:LeftUp()
		if self.LeftPressed then
		self.LeftPressed = false
		if self.Moving then -- we where moving the button!
			self:UpdateMoving()
		else -- we are pressing the button!
			self:SetButtonMouseIn()
			if self.MenuOpen then
				self:CloseMenu()
			else
				self:OpenMenu()
			end			
		end
		end
	end
	
	function MainButton.Event:LeftUpoutside()
		if self.LeftPressed then
		self.LeftPressed = false
		if self.Moving then -- mouse pointer have been faster than the button!
			self:UpdateMoving()
		end
		end
	end
	
	function MainButton.Event:MouseIn()
		if RWConfig.HermesButton.OpenOnMouseOver and (not self.MenuOpen) and (not self.Moving) then self:OpenMenu() end
		if self.LeftPressed and (not self.Moving) then
			self:SetButtonDown()
		else
			self:SetButtonMouseIn()
		end
	end

	
	-- Initialisation ----	
	
	function MainButton:UpdateMoving()
		RWConfig.HermesButton.Location.x = self:GetLeft()
		RWConfig.HermesButton.Location.y = self:GetTop()
		self.Moving = false
		InitData()
		for i = 1, self.MaxElement do self.Element[i]:InitPosition() end
		RWRoot.HermesButton.InfoFrame:AdjustPosition()
		if self.MenuWasOpen then self:OpenMenu() end
	end
	
	function MainButton:UpdateChanges()
		InitData()
		self:SetHeight(RWRoot.HermesButton.IconSize)
		self:SetWidth(RWRoot.HermesButton.IconSize)
		-- Button position
		self.MenuMaxDistance = self.MaxElement * (RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation)
		local maxDistance = self.MenuMaxDistance
		for i = 1, self.MaxElement do
			local x = i*RWRoot.HermesButton.dy
			self.Element[i].ElementParent:ClearPoint("TOPLEFT")
			self.Element[i].ElementParent:SetPoint("TOPLEFT", self, "TOPLEFT", 0, x)
			self.Element[i]:UpdateChanges()
			self.Element[i].AnimLimit = i * (RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation) * RWConst.HermesButton.AnimTime / self.MenuMaxDistance
			self.Element[i]:InitPosition()
			x = math.sqrt(x^2 + self.Element[i].MenuMaxDistance^2)
			if x > maxDistance then maxDistance = x end			
		end
		self.MaxDistance = maxDistance
		-- Update Anim limits
		for i = 1, self.MaxElement do
			x = i*(RWRoot.HermesButton.IconSize + RWConst.HermesButton.IconSeparation)
			self.Element[i]:UpdateModAnimLimit(x, self.MaxDistance)
		end
	end
	
	
	function MainButton:Init()
		self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RWConfig.HermesButton.Location.x, RWConfig.HermesButton.Location.y)
		self:UpdateChanges()
		RWRoot.HermesButton.InfoFrame:AdjustPosition()
		if RWConfig.HermesButton.Active then self:SetVisible(true) end
	end

	RWRoot.HermesButton.InfoFrame = buildInfoFrame(MainButton)
	RWRoot.HermesButton.MainButton = MainButton
end


function RWRoot.HermesButton.InitButton()

	RWRoot.HermesButton.MainButton:Init()
end


-- Activate/Desactivate ----------------------------------------------------


function RWRoot.HermesButton.Activate()
	if not RWConfig.HermesButton.Active then
		RWConfig.HermesButton.Active = true
		RWRoot.HermesButton.MainButton:SetVisible(true)	
	end
end

function RWRoot.HermesButton.Desactivate()
	if RWConfig.HermesButton.Active then
		RWConfig.HermesButton.Active = false
		RWRoot.HermesButton.MainButton:SetVisible(false)
		if RWRoot.HermesButton.MainButton.MenuOpen then	RWRoot.HermesButton.MainButton:CloseMenu() end
	end
end


-- launching ! -----------------------------------------------------------

InitData()
buildMainButton()


--	Button registration ----------------------

RWRoot.HermesButton.MainButton:RegisterMenuButton(RWRoot.locTxt.abTitle, 
	"", RWRoot.locTxt.abHermesCfgHelp, {
		"Resources\\Button\\HermesCfgUp.png",
		"Resources\\Button\\HermesCfgIn.png",
		"Resources\\Button\\HermesCfgDown.png"})

	